// HapticDLL.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"

// engine specific header file
#define DLL_USE	// always define before including adll.h
#include "adll.h"
#include "haptics/haptics.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	// make sure the library is linked
	engine_bind();
    return TRUE;
}


DLLFUNC void tr1_dll_createHaptics()
{
    haptics::create();
}


DLLFUNC void tr1_dll_destroyHaptics()
{
    if (haptics::instance())
        haptics::instance()->destroy();
}


DLLFUNC void tr1_dll_startHaptics()
{
    if (haptics::instance())
        haptics::instance()->start();
}


// Called each graphics cycle to trigger async between threads
DLLFUNC void tr1_dll_syncHaptics()
{
    if (haptics::instance())
        haptics::instance()->syncFromServo();
}

// Get Position
DLLFUNC var tr1_dll_getXPos()
{
    double x = 0;
    double y = 0;
    double z = 0;
    if (haptics::instance())
        haptics::instance()->getDeviceTerminusPositionWC(x, y, z);

    float xf = (float) (x);

    return (_VAR(xf));
}


DLLFUNC var tr1_dll_getYPos()
{
    double x = 0;
    double y = 0;
    double z = 0;
    if (haptics::instance())
        haptics::instance()->getDeviceTerminusPositionWC(x, y, z);

    float yf = (float) (y);

    return (_VAR(yf));
}


DLLFUNC var tr1_dll_getZPos()
{
    double x = 0;
    double y = 0;
    double z = 0;
    if (haptics::instance())
        haptics::instance()->getDeviceTerminusPositionWC(x, y, z);

    float zf = (float) (z);

    return (_VAR(zf));
}


// Haptic button state
DLLFUNC var tr1_dll_isHapticButtonDepressed()
{
    bool l_state = false;
    if (haptics::instance())
        l_state = haptics::instance()->isDeviceSwitchDown();

    return (_VAR(l_state));
}


//*************************************************
// clear all the current objects
//*************************************************
DLLFUNC void tr1_dll_resetObjects ()
{
	if (haptics::instance())
		haptics::instance()->resetObjects();
}


//******************************************************
// define a temporary surface, with its position
//
// IN: a_posX/Y/Z = the position of the surface
//******************************************************
DLLFUNC void tr1_dll_setSurfacePos (var a_posX, var a_posY, var a_posZ)
{
	float l_posX = _FLOAT(a_posX);
	float l_posY = _FLOAT(a_posY);
	float l_posZ = _FLOAT(a_posZ);

    if (haptics::instance())
        haptics::instance()->setSurfacePos(l_posX, l_posY, l_posZ);
}


//******************************************************
// set the normal of the current temporary surface
//
// IN: a_normX/Y/Z = the surface normal
//******************************************************
DLLFUNC void tr1_dll_setSurfaceNormal (var a_normX, var a_normY, var a_normZ)
{
	float l_normX = _FLOAT(a_normX);
	float l_normY = _FLOAT(a_normY);
	float l_normZ = _FLOAT(a_normZ);

    if (haptics::instance())
        haptics::instance()->setSurfaceNormal(l_normX, l_normY, l_normZ);
}


//******************************************************
// set the stiffness of the current temporary surface
//
// IN: a_stiffness = the stiffness to use
//******************************************************
DLLFUNC void tr1_dll_setSurfaceStiffness (var a_stiffness)
{
	float l_stiffness = _FLOAT(a_stiffness);

    if (haptics::instance())
        haptics::instance()->setSurfaceStiffness(l_stiffness);
}


//******************************************************
// define a temporary sphere
//
// IN: a_posX/Y/Z = the position of the sphere
//     a_radius = the radius of the sphere
//******************************************************
DLLFUNC void tr1_dll_setSphere (var a_posX, var a_posY, var a_posZ, var a_radius)
{
	float l_posX = _FLOAT(a_posX);
	float l_posY = _FLOAT(a_posY);
	float l_posZ = _FLOAT(a_posZ);
	float l_radius = _FLOAT(a_radius);

    if (haptics::instance())
        haptics::instance()->setSphere(l_posX, l_posY, l_posZ, l_radius);
}


//******************************************************
// set the stiffness of the current temporary sphere
//
// IN: a_stiffness = the stiffness to use
//******************************************************
DLLFUNC void tr1_dll_setSphereStiffness (var a_stiffness)
{
	float l_stiffness = _FLOAT(a_stiffness);

    if (haptics::instance())
        haptics::instance()->setSphereStiffness(l_stiffness);
}


//******************************************************
// define a temporary box, with its position
//
// IN: a_posX/Y/Z = the position of the box
//******************************************************
DLLFUNC void tr1_dll_setBoxPos (var a_posX, var a_posY, var a_posZ)
{
	float l_posX = _FLOAT(a_posX);
	float l_posY = _FLOAT(a_posY);
	float l_posZ = _FLOAT(a_posZ);

    if (haptics::instance())
        haptics::instance()->setBoxPos(l_posX, l_posY, l_posZ);
}


//******************************************************
// set the heading vector of the current temporary box
//
// IN: a_vecX/Y/Z = the heading vector
//******************************************************
DLLFUNC void tr1_dll_setBoxHeading (var a_vecX, var a_vecY, var a_vecZ)
{
	float l_vecX = _FLOAT(a_vecX);
	float l_vecY = _FLOAT(a_vecY);
	float l_vecZ = _FLOAT(a_vecZ);

    if (haptics::instance())
        haptics::instance()->setBoxHeading(l_vecX, l_vecY, l_vecZ);
}


//******************************************************
// set the up vector of the current temporary box
//
// IN: a_vecX/Y/Z = the up vector
//******************************************************
DLLFUNC void tr1_dll_setBoxUp (var a_vecX, var a_vecY, var a_vecZ)
{
	float l_vecX = _FLOAT(a_vecX);
	float l_vecY = _FLOAT(a_vecY);
	float l_vecZ = _FLOAT(a_vecZ);

    if (haptics::instance())
        haptics::instance()->setBoxUp(l_vecX, l_vecY, l_vecZ);
}


//******************************************************
// set the size of the current temporary box
//
// IN: a_sizeX/Y/Z = the size of the box
//******************************************************
DLLFUNC void tr1_dll_setBoxSize (var a_sizeX, var a_sizeY, var a_sizeZ)
{
	float l_sizeX = _FLOAT(a_sizeX);
	float l_sizeY = _FLOAT(a_sizeY);
	float l_sizeZ = _FLOAT(a_sizeZ);

    if (haptics::instance())
        haptics::instance()->setBoxSize(l_sizeX, l_sizeY, l_sizeZ);
}


//******************************************************
// set the stiffness of the current temporary box
//
// IN: a_stiffness = the stiffness to use
//******************************************************
DLLFUNC void tr1_dll_setBoxStiffness (var a_stiffness)
{
	float l_stiffness = _FLOAT(a_stiffness);

    if (haptics::instance())
        haptics::instance()->setBoxStiffness(l_stiffness);
}


//******************************************************
// set a temporary force
//
// IN: a_vecX/Y/Z = the force vector
//     a_scale = the force scale
//******************************************************
DLLFUNC void tr1_dll_setForce (var a_vecX, var a_vecY, var a_vecZ, var a_scale)
{
	float l_vecX = _FLOAT(a_vecX);
	float l_vecY = _FLOAT(a_vecY);
	float l_vecZ = _FLOAT(a_vecZ);
	float l_scale = _FLOAT(a_scale);

    if (haptics::instance())
        haptics::instance()->setForce(l_vecX, l_vecY, l_vecZ, l_scale);
}


//************************************************************
// set the weight carried by the controller
//
// IN: weight (set to negative value for no weight)
//************************************************************
DLLFUNC void tr1_dll_setHoldingWeight (var a_ID)
{
	float l_weight = _FLOAT(a_ID);

    if (haptics::instance())
        haptics::instance()->setHoldingWeight(l_weight);
}


//************************************************************
// set the haptics device to use the constrained driving forces
//************************************************************
DLLFUNC void tr1_dll_setDrivingForces ()
{
	if (haptics::instance())
		haptics::instance()->setDrivingForces();
}


//---------------------------------------------------------------
// routines for getting .ini file values
//---------------------------------------------------------------

DLLFUNC var tr1_dll_getSurfaceStiffness()
{
	return (_VAR(SURFACE_STIFFNESS));
}


DLLFUNC var tr1_dll_getSphereStiffness()
{
	return (_VAR(SPHERE_STIFFNESS));
}


DLLFUNC var tr1_dll_getBoxStiffness()
{
	return (_VAR(BOX_STIFFNESS));
}


DLLFUNC var tr1_dll_getButtonStiffness()
{
	return (_VAR(BUTTON_STIFFNESS));
}


DLLFUNC var tr1_dll_getButtonClickStiffness()
{
	return (_VAR(BUTTON_CLICK_STIFFNESS));
}


DLLFUNC var tr1_dll_getToolPosOffsetX_Sandbox()
{
	return (_VAR(TOOL_SANDBOX_OFFSET_POS_X));
}


DLLFUNC var tr1_dll_getToolPosOffsetY_Sandbox()
{
	return (_VAR(TOOL_SANDBOX_OFFSET_POS_Y));
}


DLLFUNC var tr1_dll_getToolPosOffsetZ_Sandbox()
{
	return (_VAR(TOOL_SANDBOX_OFFSET_POS_Z));
}


DLLFUNC var tr1_dll_getToolMovementScaleBase_Sandbox()
{
	return (_VAR(TOOL_SANDBOX_MOVEMENT_SCALE_BASE));
}


DLLFUNC var tr1_dll_getToolMovementScaleZ_Sandbox()
{
	return (_VAR(TOOL_SANDBOX_MOVEMENT_SCALE_Z));
}


DLLFUNC var tr1_dll_getToolPosOffsetX_Driving()
{
	return (_VAR(TOOL_DRIVING_OFFSET_POS_X));
}


DLLFUNC var tr1_dll_getToolPosOffsetY_Driving()
{
	return (_VAR(TOOL_DRIVING_OFFSET_POS_Y));
}


DLLFUNC var tr1_dll_getToolPosOffsetZ_Driving()
{
	return (_VAR(TOOL_DRIVING_OFFSET_POS_Z));
}


DLLFUNC var tr1_dll_getToolMovementScaleBase_Driving()
{
	return (_VAR(TOOL_DRIVING_MOVEMENT_SCALE_BASE));
}


DLLFUNC var tr1_dll_getToolMovementScaleZ_Driving()
{
	return (_VAR(TOOL_DRIVING_MOVEMENT_SCALE_Z));
}


DLLFUNC var tr1_dll_getSandboxCameraDistBehind()
{
	return (_VAR(SANDBOX_CAMERA_DIST_BEHIND));
}


DLLFUNC var tr1_dll_getSandboxDistAbove()
{
	return (_VAR(SANDBOX_CAMERA_DIST_ABOVE));
}


DLLFUNC var tr1_dll_getCarCamera1stDistBehind()
{
	return (_VAR(CAR_CAMERA_1ST_DIST_BEHIND));
}


DLLFUNC var tr1_dll_getCarCamera1stDistAbove()
{
	return (_VAR(CAR_CAMERA_1ST_DIST_ABOVE));
}


DLLFUNC var tr1_dll_getCarCamera3rdDistBehind()
{
	return (_VAR(CAR_CAMERA_3RD_DIST_BEHIND));
}


DLLFUNC var tr1_dll_getCarCamera3rdDistAbove()
{
	return (_VAR(CAR_CAMERA_3RD_DIST_ABOVE));
}


DLLFUNC var tr1_dll_getCameraMoveTime()
{
	return (_VAR(CAMERA_MOVE_TIME));
}


DLLFUNC var tr1_dll_getPhysicsGravity()
{
	return (_VAR(PHYSICS_GRAVITY));
}


DLLFUNC var tr1_dll_getObjHoldingVelocityScale()
{
	return (_VAR(OBJ_HOLDING_VELOCITY_SCALE));
}


DLLFUNC var tr1_dll_getFlowerpotPhysicsMass()
{
	return (_VAR(FLOWERPOT_PHYSICS_MASS));
}


DLLFUNC var tr1_dll_getFlowerpotPhysicsFriction()
{
	return (_VAR(FLOWERPOT_PHYSICS_FRICTION));
}


DLLFUNC var tr1_dll_getFlowerpotPhysicsDamping()
{
	return (_VAR(FLOWERPOT_PHYSICS_DAMPING));
}


DLLFUNC var tr1_dll_getFlowerpotScale()
{
	return (_VAR(FLOWERPOT_SCALE));
}


DLLFUNC var tr1_dll_getFlowerpotCollidePickupSize()
{
	return (_VAR(FLOWERPOT_COLLIDE_PICKUP_SIZE));
}


DLLFUNC var tr1_dll_getFlowerpotCollideNearSize()
{
	return (_VAR(FLOWERPOT_COLLIDE_NEAR_SIZE));
}


DLLFUNC var tr1_dll_getFlowerpotHapticsWeight()
{
	return (_VAR(FLOWERPOT_HAPTICS_WEIGHT));
}


DLLFUNC var tr1_dll_getFlowerpotHapticsRadius()
{
	return (_VAR(FLOWERPOT_HAPTICS_RADIUS));
}


DLLFUNC var tr1_dll_getCratePhysicsMass()
{
	return (_VAR(CRATE_PHYSICS_MASS));
}


DLLFUNC var tr1_dll_getCratePhysicsFriction()
{
	return (_VAR(CRATE_PHYSICS_FRICTION));
}


DLLFUNC var tr1_dll_getCratePhysicsDamping()
{
	return (_VAR(CRATE_PHYSICS_DAMPING));
}


DLLFUNC var tr1_dll_getCrateScale()
{
	return (_VAR(CRATE_SCALE));
}


DLLFUNC var tr1_dll_getCrateCollidePickupSize()
{
	return (_VAR(CRATE_COLLIDE_PICKUP_SIZE));
}


DLLFUNC var tr1_dll_getCrateCollideNearSize()
{
	return (_VAR(CRATE_COLLIDE_NEAR_SIZE));
}


DLLFUNC var tr1_dll_getCrateHapticsWeight()
{
	return (_VAR(CRATE_HAPTICS_WEIGHT));
}


DLLFUNC var tr1_dll_getCrateHapticsSizeX()
{
	return (_VAR(CRATE_HAPTICS_SIZE_X));
}


DLLFUNC var tr1_dll_getCrateHapticsSizeY()
{
	return (_VAR(CRATE_HAPTICS_SIZE_Y));
}


DLLFUNC var tr1_dll_getCrateHapticsSizeZ()
{
	return (_VAR(CRATE_HAPTICS_SIZE_Z));
}


DLLFUNC var tr1_dll_getCarFrontAxisDist()
{
	return (_VAR(CAR_FRONT_AXIS_DIST));
}


DLLFUNC var tr1_dll_getCarBackAxisDist()
{
	return (_VAR(CAR_BACK_AXIS_DIST));
}


DLLFUNC var tr1_dll_getCarWheelLatDistL()
{
	return (_VAR(CAR_WHEEL_LAT_DIST_L));
}


DLLFUNC var tr1_dll_getCarWheelLatDistR()
{
	return (_VAR(CAR_WHEEL_LAT_DIST_R));
}


DLLFUNC var tr1_dll_getCarWheelHeight()
{
	return (_VAR(CAR_WHEEL_HEIGHT));
}


DLLFUNC var tr1_dll_getCarPhysicsMassWheel()
{
	return (_VAR(CAR_PHYSICS_MASS_WHEEL));
}


DLLFUNC var tr1_dll_getCarPhysicsMassChassis()
{
	return (_VAR(CAR_PHYSICS_MASS_CHASSIS));
}


DLLFUNC var tr1_dll_getCarPhysicsFrictionWheel()
{
	return (_VAR(CAR_PHYSICS_FRICTION_WHEEL));
}


DLLFUNC var tr1_dll_getCarPhysicsFrictionChassis()
{
	return (_VAR(CAR_PHYSICS_FRICTION_CHASSIS));
}


DLLFUNC var tr1_dll_getCarPhysicsDampingWheel()
{
	return (_VAR(CAR_PHYSICS_DAMPING_WHEEL));
}


DLLFUNC var tr1_dll_getCarPhysicsDampingChassis()
{
	return (_VAR(CAR_PHYSICS_DAMPING_CHASSIS));
}


DLLFUNC var tr1_dll_getCarPhysicsSuspensionERP()
{
	return (_VAR(CAR_PHYSICS_SUSPENSION_ERP));
}


DLLFUNC var tr1_dll_getCarPhysicsSuspensionCFM()
{
	return (_VAR(CAR_PHYSICS_SUSPENSION_CFM));
}


DLLFUNC var tr1_dll_getCarPhysicsMaxTorque()
{
	return (_VAR(CAR_PHYSICS_MAX_TORQUE));
}


DLLFUNC var tr1_dll_getCarPhysicsMaxTorqueBrake()
{
	return (_VAR(CAR_PHYSICS_MAX_TORQUE_BRAKE));
}


DLLFUNC var tr1_dll_getCarPhysicsSteerScale()
{
	return (_VAR(CAR_PHYSICS_STEER_SCALE));
}


DLLFUNC var tr1_dll_getCarPhysicsMaxSteer()
{
	return (_VAR(CAR_PHYSICS_MAX_STEER));
}


DLLFUNC var tr1_dll_getCarPhysicsDeadzoneSteer()
{
	return (_VAR(CAR_PHYSICS_DEADZONE_STEER));
}


DLLFUNC var tr1_dll_getCarPhysicsThrottleScale()
{
	return (_VAR(CAR_PHYSICS_THROTTLE_SCALE));
}


DLLFUNC var tr1_dll_getCarPhysicsMaxThrottle()
{
	return (_VAR(CAR_PHYSICS_MAX_THROTTLE));
}


DLLFUNC var tr1_dll_getCarPhysicsDeadzoneThrottle()
{
	return (_VAR(CAR_PHYSICS_DEADZONE_THROTTLE));
}


DLLFUNC var tr1_dll_getCarUprightSpringScale()
{
	return (_VAR(CAR_UPRIGHT_SPRING_SCALE));
}


DLLFUNC var tr1_dll_getCarHoldingVelocityScale()
{
	return (_VAR(CAR_HOLDING_VELOCITY_SCALE));
}


DLLFUNC var tr1_dll_getCarCollidePickupSize()
{
	return (_VAR(CAR_COLLIDE_PICKUP_SIZE));
}


DLLFUNC var tr1_dll_getCarCollideNearSize()
{
	return (_VAR(CAR_COLLIDE_NEAR_SIZE));
}


DLLFUNC var tr1_dll_getCarHapticsWeight()
{
	return (_VAR(CAR_HAPTICS_WEIGHT));
}


DLLFUNC var tr1_dll_getCarHapticsSizeX()
{
	return (_VAR(CAR_HAPTICS_SIZE_X));
}


DLLFUNC var tr1_dll_getCarHapticsSizeY()
{
	return (_VAR(CAR_HAPTICS_SIZE_Y));
}


DLLFUNC var tr1_dll_getCarHapticsSizeZ()
{
	return (_VAR(CAR_HAPTICS_SIZE_Z));
}


DLLFUNC var tr1_dll_getChassisImpactThreshold()
{
	return (_VAR(CHASSIS_IMPACT_THRESHOLD));
}


DLLFUNC var tr1_dll_getChassisImpactScale()
{
	return (_VAR(CHASSIS_IMPACT_SCALE));
}


DLLFUNC var tr1_dll_getWheelImpactThreshold()
{
	return (_VAR(WHEEL_IMPACT_THRESHOLD));
}


DLLFUNC var tr1_dll_getWheelImpactScale()
{
	return (_VAR(WHEEL_IMPACT_SCALE));
}


DLLFUNC var tr1_dll_getShrinkBtnSandboxX()
{
	return (_VAR(SHRINK_BTN_SANDBOX_X));
}


DLLFUNC var tr1_dll_getShrinkBtnSandboxY()
{
	return (_VAR(SHRINK_BTN_SANDBOX_Y));
}


DLLFUNC var tr1_dll_getShrinkBtnSandboxZ()
{
	return (_VAR(SHRINK_BTN_SANDBOX_Z));
}


DLLFUNC var tr1_dll_getShrinkBtnScale()
{
	return (_VAR(SHRINK_BTN_SCALE));
}


DLLFUNC var tr1_dll_getShrinkBtnCollideSize()
{
	return (_VAR(SHRINKBTN_COLLIDE_SIZE));
}


DLLFUNC var tr1_dll_getShrinkBtnNearSize()
{
	return (_VAR(SHRINKBTN_NEAR_SIZE));
}


DLLFUNC var tr1_dll_getShrinkBtnHapticsSize()
{
	return (_VAR(SHRINKBTN_HAPTICS_SIZE));
}


DLLFUNC var tr1_dll_getGrowBtnSandboxX()
{
	return (_VAR(GROW_BTN_SANDBOX_X));
}


DLLFUNC var tr1_dll_getGrowBtnSandboxY()
{
	return (_VAR(GROW_BTN_SANDBOX_Y));
}


DLLFUNC var tr1_dll_getGrowBtnSandboxZ()
{
	return (_VAR(GROW_BTN_SANDBOX_Z));
}


DLLFUNC var tr1_dll_getGrowBtnScale()
{
	return (_VAR(GROW_BTN_SCALE));
}


DLLFUNC var tr1_dll_getGrowBtnCollideSize()
{
	return (_VAR(GROWBTN_COLLIDE_SIZE));
}


DLLFUNC var tr1_dll_getGrowBtnNearSize()
{
	return (_VAR(GROWBTN_NEAR_SIZE));
}


DLLFUNC var tr1_dll_getGrowBtnHapticsSize()
{
	return (_VAR(GROWBTN_HAPTICS_SIZE));
}


DLLFUNC var tr1_dll_getBtnMinScale()
{
	return (_VAR(BTN_MIN_SCALE));
}


DLLFUNC var tr1_dll_getBtnClickScale()
{
	return (_VAR(BTN_CLICK_SCALE));
}
