// TENEOSTD
//////////////////////////////////////////////////////////////////////
// haptics.h: interface for the haptics class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(JAV_HAPTICS_H)
#define JAV_HAPTICS_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <hdl/hdl.h>
#include <hdlu/hdlu.h>
#include <fstream>
#include "vector.h"
#include "matrix.h"



// A few useful constants
const bool bBlocking = true;
const bool bNonBlocking = false;

#define iniFileName "tr1_haptics.ini"

// general values
extern int    IS_FALCON;
extern double TIME_SCALE;

//---- general purpose vars (don't need to be tuned for each haptics device)----

// camera values
extern float SANDBOX_CAMERA_DIST_BEHIND;		// distance from car to place sandbox camera when returning from driving
extern float SANDBOX_CAMERA_DIST_ABOVE;

extern float CAR_CAMERA_1ST_DIST_BEHIND;		// distance from car to place driving camera (1st person camera)
extern float CAR_CAMERA_1ST_DIST_ABOVE;

extern float CAR_CAMERA_3RD_DIST_BEHIND;		// distance from car to place driving camera (3rd person camera)
extern float CAR_CAMERA_3RD_DIST_ABOVE;

extern float CAMERA_MOVE_TIME;					// time the camera sandbox/driving transition takes

// general game physics values
extern float PHYSICS_GRAVITY;					// gravity

// generic physics object values
extern float OBJ_HOLDING_VELOCITY_SCALE;		// velocity scale to use when holding and dragging the object

// flowerpot physics object values
extern float FLOWERPOT_PHYSICS_MASS;			// mass of the object
extern float FLOWERPOT_PHYSICS_FRICTION;		// friction applied to the object
extern float FLOWERPOT_PHYSICS_DAMPING;			// amount of damping

extern float FLOWERPOT_SCALE;					// display scale
extern float FLOWERPOT_COLLIDE_PICKUP_SIZE;		// pickup size of the object
extern float FLOWERPOT_COLLIDE_NEAR_SIZE;		// near (run haptics) size of the object
extern float FLOWERPOT_HAPTICS_RADIUS;			// radius of the haptics sphere

// crate physics object values
extern float CRATE_PHYSICS_MASS;				// mass of the object
extern float CRATE_PHYSICS_FRICTION;			// friction applied to the object
extern float CRATE_PHYSICS_DAMPING;				// amount of damping

extern float CRATE_SCALE;						// display scale
extern float CRATE_COLLIDE_PICKUP_SIZE;			// pickup size of the object
extern float CRATE_COLLIDE_NEAR_SIZE;			// near (run haptics) size of the object

extern float CRATE_HAPTICS_SIZE_X;				// haptics box size
extern float CRATE_HAPTICS_SIZE_Y;
extern float CRATE_HAPTICS_SIZE_Z;

// Car Physics constants
extern float CAR_FRONT_AXIS_DIST;				// distance from car center to front axle
extern float CAR_BACK_AXIS_DIST;				// distance from car center to back axle
extern float CAR_WHEEL_LAT_DIST_L;				// distance out from car center to wheel (left wheel)
extern float CAR_WHEEL_LAT_DIST_R;				// distance out from car center to wheel (right wheel)
extern float CAR_WHEEL_HEIGHT;					// height of wheels relative to car center

extern float CAR_PHYSICS_MASS_WHEEL;			// mass per wheel
extern float CAR_PHYSICS_MASS_CHASSIS;			// mass for chassis lighter than wheels so it doesn't tilt

extern float CAR_PHYSICS_FRICTION_WHEEL;		// friction of wheels
extern float CAR_PHYSICS_FRICTION_CHASSIS;		// friction of chassis (low to avoid getting stuck on ramps)

extern float CAR_PHYSICS_DAMPING_WHEEL;			// damping ang for wheels
extern float CAR_PHYSICS_DAMPING_CHASSIS;		// damping lin/ang for chassis

extern float CAR_PHYSICS_SUSPENSION_ERP;		// how quickly shocks are reduced to correct position (100,000 max, 0: never)
extern float CAR_PHYSICS_SUSPENSION_CFM;		// how far shocks can overshoot (0: never, 100,000 max)

extern float CAR_PHYSICS_MAX_TORQUE;			// maximum torque applied to wheels
extern float CAR_PHYSICS_MAX_TORQUE_BRAKE;		// maximum torque when braking

extern float CAR_PHYSICS_STEER_SCALE;			// steering scale factor
extern float CAR_PHYSICS_MAX_STEER;				// max steer angle
extern float CAR_PHYSICS_DEADZONE_STEER;		// dead-zone at center of haptics control

extern float CAR_PHYSICS_THROTTLE_SCALE;		// throttle scale factor
extern float CAR_PHYSICS_MAX_THROTTLE;			// max throttle
extern float CAR_PHYSICS_DEADZONE_THROTTLE;		// dead-zone at center of haptics control, throttle

extern float CAR_UPRIGHT_SPRING_SCALE;			// scale factor for the spring that keeps the car upright

extern float CAR_HOLDING_VELOCITY_SCALE;		// velocity scale to use when holding the car

extern float CAR_COLLIDE_PICKUP_SIZE;			// pickup collision size
extern float CAR_COLLIDE_NEAR_SIZE;				// near to cursor haptics threshold
extern float CAR_HAPTICS_SIZE_X;				// haptics box size
extern float CAR_HAPTICS_SIZE_Y;
extern float CAR_HAPTICS_SIZE_Z;

// button values
extern float SHRINK_BTN_SANDBOX_X;				// shrink button screen position
extern float SHRINK_BTN_SANDBOX_Y;
extern float SHRINK_BTN_SANDBOX_Z;

extern float SHRINK_BTN_SCALE;					// shrink button display scale
extern float SHRINKBTN_COLLIDE_SIZE;			// click distance
extern float SHRINKBTN_NEAR_SIZE;				// near (run haptics) size of the object
extern float SHRINKBTN_HAPTICS_SIZE;			// haptics radius

extern float GROW_BTN_SANDBOX_X;				// grow button screen position
extern float GROW_BTN_SANDBOX_Y;
extern float GROW_BTN_SANDBOX_Z;

extern float GROW_BTN_SCALE;					// grow button display scale
extern float GROWBTN_COLLIDE_SIZE;				// click distance
extern float GROWBTN_NEAR_SIZE;					// near (run haptics) size of the object
extern float GROWBTN_HAPTICS_SIZE;				// haptics radius

extern float BTN_MIN_SCALE;						// minimum scale of the button (adjusted from 0..1.0)
extern float BTN_CLICK_SCALE;					// click scale of the button (adjusted from 0..1.0)


//---- haptics vars that can be tuned for each type of device----

// haptics object stiffnesses
extern double SURFACE_STIFFNESS;				// floor surfaces
extern double SPHERE_STIFFNESS;					// sphere haptics object
extern double BOX_STIFFNESS;					// box haptics object
extern double DRIVING_STIFFNESS;				// driving constraint
extern double BUTTON_STIFFNESS;					// button
extern double BUTTON_CLICK_STIFFNESS;			// button when clicked

// sandbox view values
extern float TOOL_SANDBOX_OFFSET_POS_X;			// Offset the X position of the hand tool, sandbox view
extern float TOOL_SANDBOX_OFFSET_POS_Y;			// Offset the Y position of the hand tool, sandbox view
extern float TOOL_SANDBOX_OFFSET_POS_Z;			// Offset the Z position of the hand tool, sandbox view
extern float TOOL_SANDBOX_MOVEMENT_SCALE_BASE;	// Scale the base movement of the hand tool, sandbox view
extern float TOOL_SANDBOX_MOVEMENT_SCALE_Z;		// additional Z Scale of the hand tool, sandbox view

// driving view values
extern float TOOL_DRIVING_OFFSET_POS_X;			// Offset the X position of the hand tool, driving view
extern float TOOL_DRIVING_OFFSET_POS_Y;			// Offset the Y position of the hand tool, driving view
extern float TOOL_DRIVING_OFFSET_POS_Z;			// Offset the Z position of the hand tool, driving view
extern float TOOL_DRIVING_MOVEMENT_SCALE_BASE;	// Scale the base movement of the hand tool, driving view
extern float TOOL_DRIVING_MOVEMENT_SCALE_Z;		// additional Z Scale of the hand tool, driving view

extern float DRIVING_CONSTRAINT_OFFSET_X;		// driving mode constrained forces offset
extern float DRIVING_CONSTRAINT_OFFSET_Y;
extern float DRIVING_CONSTRAINT_OFFSET_Z;

extern float DRIVING_CONSTRAINT_SCALE_X;		// driving mode constrained forces scale
extern float DRIVING_CONSTRAINT_SCALE_Y;
extern float DRIVING_CONSTRAINT_SCALE_Z;

extern float CHASSIS_IMPACT_THRESHOLD;			// speed threshold that causes the chassis to generate a haptics impact force
extern float CHASSIS_IMPACT_SCALE;				// scale to apply to chassis impact haptics forces

extern float WHEEL_IMPACT_THRESHOLD;			// speed threshold that causes a wheel to generate a haptics impact force
extern float WHEEL_IMPACT_SCALE;				// scale to apply to wheel impact haptics forces

// object weights
extern float FLOWERPOT_HAPTICS_WEIGHT;			// haptics weight
extern float CRATE_HAPTICS_WEIGHT;				// haptics weight
extern float CAR_HAPTICS_WEIGHT;				// the haptics pickup weight



// number of surfaces/objects available
#define NUM_SURFACES	20
#define NUM_SPHERES		20
#define NUM_BOXES		20
#define NUM_FORCES		20


class haptics  
{
    friend HDLServoOpExitCode ContactCB(void *data);
    friend HDLServoOpExitCode GetStateCB(void *pUsrData);

public:
  
    static haptics* instance();
    static void create();
    static void destroy();
    void start();

    void initParams();
    double getParam(std::string a_paramName, double a_default);
    int getParam(std::string a_paramName, int a_default);

    // Destructor
    virtual ~haptics();

    // Sync data returned by hapics object to the servo thread in 
    // a thread safe manner.  Call only once per application frame
    // (usually at the beginning of the graphics draw).
    void syncFromServo();

    // Returns the position of the haptic device terminus in millimeters.
    // If device has not been initialized, a position of 0,0,0 is returned
    // Note that 0,0,0 is at the center of the devices workspace
    // (i.e., its reset position)
    bool getDeviceTerminusPositionWC(double &px, double &py, double &pz);

    // Returns the cumulative matrix of the haptic device terminus in ghost format
    // If device has not been initialized, 0,0,0 is returned
    bool getDeviceTerminusMatrixWC(etMatrix& cummatrix);

    // Returns the state of the device (stylus) switch -- true if depressed, false otherwise
    bool isDeviceSwitchDown();

	// Servo loop only
    etVector calculateForces();
    void doSyncOp();

    //
    // Game specific functions
    //

	// reset all the haptics objects for the next update frame
	void resetObjects ()
	{
		m_numSurfaces = 0;
		m_numSpheres = 0;
		m_numBoxes = 0;
		m_numForces = 0;
		m_bDoDrivingForces = 0;
	}

	// create a surface at a position
	// NOTE: setNormalSurface must be called for this surface to set the normal
	void setSurfacePos (double pos_x, double pos_y, double pos_z)
	{
		if (m_numSurfaces < NUM_SURFACES)
		{	// still have room for more surfaces
			m_surfaces[m_numSurfaces].pos = etVector(pos_x, pos_y, pos_z);
			m_surfaces[m_numSurfaces].normal = etVector(0.0, 1.0, 0.0);
			m_surfaces[m_numSurfaces].stiffness = SURFACE_STIFFNESS;

			m_numSurfaces++;
		}
	}

    // set the normal of the surface last created by setSurfacePos
	void setSurfaceNormal (double norm_x, double norm_y, double norm_z)
	{
		m_surfaces[m_numSurfaces-1].normal = etVector(norm_x, norm_y, norm_z);
	}

	//set the stiffness of the last surface created by setSurfacePos
	void setSurfaceStiffness (double stiffness)
	{
		m_surfaces[m_numSurfaces-1].stiffness = stiffness;
	}

    // create a haptics sphere
	void setSphere (double pos_x, double pos_y, double pos_z, double radius)
	{
		if (m_numSpheres < NUM_SPHERES)
		{	// still have room for more spheres
			m_spheres[m_numSpheres].pos = etVector(pos_x, pos_y, pos_z);
			m_spheres[m_numSpheres].radius = radius;
			m_spheres[m_numSpheres].stiffness = SPHERE_STIFFNESS;
			
			m_numSpheres++;
		}
	}

	//set the stiffness of the last sphere created by setSphere
	void setSphereStiffness (double stiffness)
	{
		m_spheres[m_numSpheres-1].stiffness = stiffness;
	}

	// create a box at a position
	void setBoxPos (double pos_x, double pos_y, double pos_z)
	{
		if (m_numBoxes < NUM_BOXES)
		{	// still have room for more boxes
			m_boxes[m_numBoxes].pos = etVector(pos_x, pos_y, pos_z);
			m_boxes[m_numBoxes].size = etVector(10.0, 10.0, 10.0);
			m_boxes[m_numBoxes].headVec = etVector(0.0, 0.0, 1.0);
			m_boxes[m_numBoxes].upVec = etVector(0.0, 1.0, 0.0);
			m_boxes[m_numBoxes].stiffness = BOX_STIFFNESS;

			m_numBoxes++;
		}
	}

    // set the heading vector of the box last created by setBoxPos
	void setBoxHeading (double vec_x, double vec_y, double vec_z)
	{
		m_boxes[m_numBoxes-1].headVec = etVector(vec_x, vec_y, vec_z);
	}

    // set the up vector of the box last created by setBoxPos
	void setBoxUp (double vec_x, double vec_y, double vec_z)
	{
		m_boxes[m_numBoxes-1].upVec = etVector(vec_x, vec_y, vec_z);
	}

    // set the size of the box last created by setBoxPos
	void setBoxSize (double size_x, double size_y, double size_z)
	{
		m_boxes[m_numBoxes-1].size = etVector(size_x, size_y, size_z);
	}

	//set the stiffness of the last box created by setSurfacePos
	void setBoxStiffness (double stiffness)
	{
		m_boxes[m_numBoxes-1].stiffness = stiffness;
	}

	// set a temporary force
	void setForce (double dir_x, double dir_y, double dir_z, double force)
	{
		if (m_numForces < NUM_FORCES)
		{	// still have room for more forces
			m_forces[m_numForces].dir = etVector(dir_x, dir_y, dir_z);
			m_forces[m_numForces].force = force;

			m_numForces++;
		}
	}
	
	// set the holding weight
	void setHoldingWeight (double a_weight)
	{
		m_holdingWeight = a_weight;
	}

	// turn on the driving mode constrained forces
	void setDrivingForces ()
	{
		m_bDoDrivingForces = 1;
	}
	
	// Create a haptic sphere at a desired position, radius
    etVector doSphereAt(etVector a_position, double a_radius, etVector a_toolPos, double a_stiffness);

	// create a haptics oriented box
	etVector doBoxAt(etVector a_position, etVector a_size, etVector a_headVec, etVector a_upVec, etVector a_toolPos, double a_stiffness);

	// create constrained driving effects
	etVector haptics::doDrivingForces(etVector a_toolPos);

private:

      // Constructor
    haptics();

    static haptics* s_instance;

    // Does first order haptic scene initialization.
    // Note that the haptic device is not  reset
    // or added to the scene.  addDevice() must be
    // called to do this.
    bool initialize();
 

    // Servo thead end-effector state
    etVector m_terminusPositionServo;
    etMatrix m_terminusTransformServo;
    bool m_isButtonDownServo;
    double m_deviceWorkspaceDim[6];
	etVector m_objectVelocityServo;


    // Client thread end-effector state
    etVector m_terminusPositionClient;
    etMatrix m_terminusTransformClient;
    bool m_isButtonDownClient;
    HDLOpHandle m_hContactCB;

   	bool m_isFirstTime;
    double m_lastTime;

    bool m_isServoRunning;

    // Client game vars

	// data struct for holding info about a surface
	struct SurfaceStruct
	{
		etVector pos;		// the position of the surface
		etVector normal;	// the orientation of the surface

		int surfaceType;	// the type of surface

		double stiffness;	// stiffness of the surface
	};

	// the list of current surfaces
	SurfaceStruct m_surfaces[NUM_SURFACES];
	int m_numSurfaces;

	// data struct for holding info about a haptic sphere
	struct SphereStruct
	{
		etVector pos;		// position of the sphere

		int radius;			// size of the sphere

		double stiffness;	// stiffness of the sphere
	};

	// list of current spheres
	SphereStruct m_spheres[NUM_SPHERES];
	int m_numSpheres;

	// data struct for holding info about a haptics box
	struct BoxStruct
	{
		etVector pos;		// position of the box

		etVector size;		// the size of the box

		etVector headVec;	// heading vector of the box
		etVector upVec;		// up vector of the box

		double stiffness;	// stiffness of the box
	};

	// list of current boxes
	BoxStruct m_boxes[NUM_BOXES];
	int m_numBoxes;

	// data struct for holding info about temporary forces
	struct ForceStruct
	{
		etVector dir;	// direction of the force

		double force;	// the force scale
	};

	ForceStruct m_forces[NUM_FORCES];
	int m_numForces;

	// weight being carried
	double m_holdingWeight;

	// flag - do constrained forces for driving mode
	int m_bDoDrivingForces;

	etVector m_prevToolPosition;

	std::ofstream m_log;
};

#endif // !defined(JAV_HAPTICS_H)
