//*******************************************************************
// camera controls
//*******************************************************************

// camera settings
define TR1_CAMERA_X, 			-1200.0;
define TR1_CAMERA_Y, 			0.0;
define TR1_CAMERA_Z, 			1000.0;

define TR1_CAMERA_PAN, 			0.0;
define TR1_CAMERA_TILT, 		-30.0;
define TR1_CAMERA_ROLL, 		0.0;

define TR1_CAMERA_ARC, 			45.0;
define TR1_CAMERA_AMBIENT, 	0;
define TR1_CAMERA_CLIP_NEAR, 	15.0;
define TR1_CAMERA_CLIP_FAR, 	50000.0;

// distance to place camera relative to car when returning to sandbox mode
var TR1_SANDBOX_CAMERA_DIST_BEHIND;
var TR1_SANDBOX_CAMERA_DIST_ABOVE;

// distance to place camera relative to car when driving (1st person camera)
var TR1_CAR_CAMERA_1ST_DIST_BEHIND;
var TR1_CAR_CAMERA_1ST_DIST_ABOVE;

// distance to place camera relative to car when driving (3rd person camera)
var TR1_CAR_CAMERA_3RD_DIST_BEHIND;
var TR1_CAR_CAMERA_3RD_DIST_ABOVE;

// time camera transition takes
var TR1_CAMERA_MOVE_TIME;

// current sandbox camera position and angle
var gtr1_SandboxCameraPos[3];
var gtr1_SandboxCameraAngle[3];

// inputs to tr1_camera_updateTransition
var gtr1_CamMoveToPos[3];
var gtr1_CamMoveToAng[3];


//*******************************************************************
// code for running camera transition
//
// IN:  v_timeLeft = time left for the transition (in ticks)
//		  gtr1_CamMoveToPos = the position to move the camera to
//		  gtr1_CamMoveToAng = the orientation to move the camera to
//
// RETURNS: new time left (if <=0, then transition is done)
//*******************************************************************
function tr1_camera_updateTransition (v_timeLeft)
{
	var l_moveVec[3];
	var l_moveDist;
	
	// update time
	v_timeLeft -= gtr1_frameTicks;
	if (v_timeLeft <= 0)
	{	// done with transition
		vec_set(camera.x, gtr1_CamMoveToPos);
		vec_set(camera.pan, gtr1_CamMoveToAng);
		
		return(v_timeLeft);
	}
	
	// adjust camera position
	
	// get the vector to move towards
	vec_diff(l_moveVec, gtr1_CamMoveToPos, camera.x);
	l_moveDist = vec_length(l_moveVec);
	vec_normalize(l_moveVec, 1);
	
	// calculate how far to move
	l_moveDist /= v_timeLeft;
	l_moveDist *= gtr1_frameTicks;	// adjust for framerate
	
	vec_scale(l_moveVec, l_moveDist);
	
	// adjust the camera pos
	vec_add(camera.x, l_moveVec);
	
	// adjust camera orientation
	
	// first, pan....
	l_moveDist = gtr1_CamMoveToAng.pan - camera.pan;
	l_moveDist /= v_timeLeft;
	l_moveDist *= gtr1_frameTicks;	// adjust for framerate
	camera.pan += l_moveDist;
	
	// then tilt....
	l_moveDist = gtr1_CamMoveToAng.tilt - camera.tilt;
	l_moveDist /= v_timeLeft;
	l_moveDist *= gtr1_frameTicks;	// adjust for framerate
	camera.tilt += l_moveDist;
	
	return(v_timeLeft);
}


//*******************************************************************
// launch the camera and update it based on its current mode
//*******************************************************************
function tr1_Camera_LaunchAndUpdate ()
{
	var l_hapticsPos[3];
	var l_camPos[3];
	
	var l_moveTimeLeft;
	
	var l_bDriving3rdPerson = 0;
	var l_bLastDrivingKey = off;
	
	var l_collideStart[3];
	var l_collideEnd[3];
	
	// inital camera position
	camera.x = TR1_CAMERA_X;
	camera.y = TR1_CAMERA_Y;    
	camera.z = TR1_CAMERA_Z;
	
	// inital camera orientation
	camera.pan = TR1_CAMERA_PAN;
	camera.tilt = TR1_CAMERA_TILT;	
	camera.roll = TR1_CAMERA_ROLL;
	
	// camera view settings
	camera.arc = TR1_CAMERA_ARC;	
	camera.ambient = TR1_CAMERA_AMBIENT;	
	camera.clip_near = TR1_CAMERA_CLIP_NEAR;
	camera.clip_far = TR1_CAMERA_CLIP_FAR;
	
	gtr1_CameraMode = TR1_CAMERA_MODE_PICKUP;

	// load the .ini file constants
	TR1_SANDBOX_CAMERA_DIST_BEHIND = tr1_dll_getSandboxCameraDistBehind();
	TR1_SANDBOX_CAMERA_DIST_ABOVE = tr1_dll_getSandboxDistAbove();
	TR1_CAR_CAMERA_1ST_DIST_BEHIND = tr1_dll_getCarCamera1stDistBehind();
	TR1_CAR_CAMERA_1ST_DIST_ABOVE = tr1_dll_getCarCamera1stDistAbove();
	TR1_CAR_CAMERA_3RD_DIST_BEHIND = tr1_dll_getCarCamera3rdDistBehind();
	TR1_CAR_CAMERA_3RD_DIST_ABOVE = tr1_dll_getCarCamera3rdDistAbove();
	TR1_CAMERA_MOVE_TIME = tr1_dll_getCameraMoveTime();
	
	while(1)
	{
		if (gtr1_microgameActive)
		{
			if (gtr1_NewCamMode > TR1_CAMERA_MODE_NONE)
			{	// new camera mode
				if (gtr1_NewCamMode == TR1_CAMERA_MODE_DRIVING)
				{
					l_bDriving3rdPerson = 0;
				}

				if (gtr1_NewCamMode == TR1_CAMERA_MODE_GOTO_DRIVING)
				{
					l_moveTimeLeft = TR1_CAMERA_MOVE_TIME;
				}

				if (gtr1_NewCamMode == TR1_CAMERA_MODE_GOTO_SANDBOX)
				{
					l_moveTimeLeft = TR1_CAMERA_MOVE_TIME;
						
					// grab the entity ptr for the car
					tr1_Object_GetEntity(gtr1_CarID);

					// set the camera pos and angle to transition to
						
					// convert the orientation of the car to a vector
					vec_for_angle(gtr1_SandboxCameraPos, gtr1_pCurIDEntity.pan);

	 				// reverse the vector to aim behind the car, and extend it
			 		vec_scale(gtr1_SandboxCameraPos, TR1_SANDBOX_CAMERA_DIST_BEHIND);
 		
					// adjust upward a bit
					gtr1_SandboxCameraPos.z += TR1_SANDBOX_CAMERA_DIST_ABOVE;
 		
					// add in the car's position
					vec_add(gtr1_SandboxCameraPos, gtr1_pCurIDEntity.x);

     				// aim the camera at the car
  					vec_diff(l_camPos, gtr1_pCurIDEntity.x, gtr1_SandboxCameraPos); 
					vec_to_angle(gtr1_SandboxCameraAngle, l_camPos);
				}
				
				gtr1_CameraMode = gtr1_NewCamMode;
				gtr1_NewCamMode = TR1_CAMERA_MODE_NONE;
			}
	
	 		// adjust camera based on mode
			if (gtr1_CameraMode == TR1_CAMERA_MODE_ROTATE)
			{	// rotating the camera
				l_hapticsPos.y = -tr1_dll_getXPos() * 0.02;
				l_hapticsPos.z = tr1_dll_getYPos() * 0.02;

				vec_set(l_camPos, camera.pan);
				l_camPos.pan += l_hapticsPos.y;
				l_camPos.tilt += l_hapticsPos.z;
		
				l_camPos.tilt = clamp(l_camPos.tilt, -50, -10);
		
				if (l_camPos.pan > 360.0)
				{
					l_camPos.pan -= 360.0;
				}
				if (l_camPos.pan < -360.0)
				{
					l_camPos += 360.0;
				}
				
				vec_set(camera.pan, l_camPos);
			}
			
			if (gtr1_CameraMode == TR1_CAMERA_MODE_ROTATE_KEYS)
			{	// using keyboard to rotate camera
				l_hapticsPos.y = 0;
				l_hapticsPos.z = 0;
			
				if (key_a == on)
				{
					l_hapticsPos.y = 1.0;
				}
				
				if (key_d == on)
				{
					l_hapticsPos.y = -1.0;
				}
				
				if (key_w == on)
				{
					l_hapticsPos.z = 1.0;
				}
				
				if (key_s == on)
				{
					l_hapticsPos.z = -1.0;
				}
				
				vec_set(l_camPos, camera.pan);
				l_camPos.pan += l_hapticsPos.y;
				l_camPos.tilt += l_hapticsPos.z;
		
				l_camPos.tilt = clamp(l_camPos.tilt, -50, -10);
		
				if (l_camPos.pan > 360.0)
				{
					l_camPos.pan -= 360.0;
				}
				if (l_camPos.pan < -360.0)
				{
					l_camPos += 360.0;
				}
				
				vec_set(camera.pan, l_camPos);
			}

			if (gtr1_CameraMode == TR1_CAMERA_MODE_STRAFE)
			{	// strafing the camera
				l_hapticsPos.x = -tr1_dll_getZPos() * 0.4;
				l_hapticsPos.y = -tr1_dll_getXPos() * 0.4;
				l_hapticsPos.z = tr1_dll_getYPos() * 0.4;
		
				// rotate pos to match camera
				vec_rotate(l_hapticsPos, camera.pan);
		
				// set new camera pos
				vec_set(l_camPos, camera.x);
				vec_add(l_camPos, l_hapticsPos);
				l_camPos.x = clamp(l_camPos.x, -5000, 5000);
				l_camPos.y = clamp(l_camPos.y, -5000, 5000);
				l_camPos.z = clamp(l_camPos.z, 500, 2000);
		
				vec_set(camera.x, l_camPos);
			}
	
			if (gtr1_CameraMode == TR1_CAMERA_MODE_STRAFE_KEYS)
			{
				l_hapticsPos.x = 0;
				l_hapticsPos.y = 0;
				l_hapticsPos.z = 0;
				
				if (key_cuu == on)
				{
					l_hapticsPos.x = 10.0;
				}
				
				if (key_cud == on)
				{
					l_hapticsPos.x = -10.0;
				}
				
				if (key_cul == on)
				{
					l_hapticsPos.y = 10.0;
				}
				
				if (key_cur == on)
				{
					l_hapticsPos.y = -10.0;
				}
				
				if (key_pgup == on)
				{
					l_hapticsPos.z = 10.0;
				}
				
				if (key_pgdn == on)
				{
					l_hapticsPos.z = -10.0;
				}
				
				// rotate pos to match camera
				vec_rotate(l_hapticsPos, camera.pan);
		
				// set new camera pos
				vec_set(l_camPos, camera.x);
				vec_add(l_camPos, l_hapticsPos);
				l_camPos.x = clamp(l_camPos.x, -5000, 5000);
				l_camPos.y = clamp(l_camPos.y, -5000, 5000);
				l_camPos.z = clamp(l_camPos.z, 500, 2000);
		
				vec_set(camera.x, l_camPos);
			}	
			
			if (gtr1_CameraMode == TR1_CAMERA_MODE_DRIVING)
			{
 				if ((key_q) && (!l_bLastDrivingKey))
 				{	// toggle the camera view between 1st and 3rd person
 					l_bDriving3rdPerson = 1 - l_bDriving3rdPerson;
 				}
 				l_bLastDrivingKey = key_q;
 				
				// grab the entity ptr for the car
				tr1_Object_GetEntity(gtr1_CarID);

 				if (l_bDriving3rdPerson == 0)
 				{	// using 1st person camera (default)
 				
 					// convert the orientation of the car to a vector
	 				vec_for_angle(l_camPos, gtr1_pCurIDEntity.pan);

	 				// reverse the vector to aim behind the car, and extend it
			 		vec_scale(l_camPos, TR1_CAR_CAMERA_1ST_DIST_BEHIND);
 		
 					// adjust upward a bit
 					l_camPos.z += TR1_CAR_CAMERA_1ST_DIST_ABOVE;
 		
 					// add in the car's position
 					vec_add(l_camPos, gtr1_pCurIDEntity.x);

			 		// set the camera to this position
 					vec_set(camera.x, l_camPos);
 					
 					// set the orientation to match the car
 					vec_set(camera.pan, gtr1_pCurIDEntity.pan);
 				}
 				else
 				{	// using 3rd person camera
 					
 					// convert the orientation of the car to a vector
	 				vec_for_angle(l_camPos, gtr1_pCurIDEntity.pan);

	 				// reverse the vector to aim behind the car, and extend it
			 		vec_scale(l_camPos, TR1_CAR_CAMERA_3RD_DIST_BEHIND);

 					// add in the car's position
 					vec_add(l_camPos, gtr1_pCurIDEntity.x);

					vec_set(l_collideStart, vector(l_camPos.x, l_camPos.y, l_camPos.z + 2000));
					vec_set(l_collideEnd, vector(l_camPos.x, l_camPos.y, l_camPos.z - 2000));
					c_trace(l_collideStart, l_collideEnd, (IGNORE_PASSABLE+IGNORE_PASSENTS+IGNORE_MODELS));
					if (trace_hit == 1)
					{	// collide ray hit terrain... set camera above terrain
						l_camPos.z = target.z + TR1_CAR_CAMERA_3RD_DIST_ABOVE;
					}
					else
					{	// shouldn't ever get here, but just in case...
	 					// adjust upward a bit
 						l_camPos.z += TR1_CAR_CAMERA_3RD_DIST_ABOVE;
 					}

			 		// set the camera to this position
 					vec_set(camera.x, l_camPos);
 		
      			// aim the camera at the car
   				vec_diff(l_camPos, gtr1_pCurIDEntity.x, camera.x); 
					vec_to_angle(camera.pan, l_camPos);
					camera.roll = 0;
				}
			}
			
			if (gtr1_CameraMode == TR1_CAMERA_MODE_GOTO_DRIVING)
			{
				// grab the entity ptr for the car
				tr1_Object_GetEntity(gtr1_CarID);

				// convert the orientation of the car to a vector
 				vec_for_angle(gtr1_CamMoveToPos, gtr1_pCurIDEntity.pan);

 				// reverse the vector to aim behind the car, and extend it
		 		vec_scale(gtr1_CamMoveToPos, TR1_CAR_CAMERA_1ST_DIST_BEHIND);
 		
				// adjust upward a bit
				gtr1_CamMoveToPos.z += TR1_CAR_CAMERA_1ST_DIST_ABOVE;
 		
				// add in the car's position
				vec_add(gtr1_CamMoveToPos, gtr1_pCurIDEntity.x);

				// set the orientation to match the car
				vec_set(gtr1_CamMoveToAng, gtr1_pCurIDEntity.pan);

				l_moveTimeLeft = tr1_camera_updateTransition(l_moveTimeLeft);
				if (l_moveTimeLeft <= 0)
				{	// finished transition... goto driving
					gtr1_CameraMode = TR1_CAMERA_MODE_DRIVING;
					
					l_bDriving3rdPerson = 0;
				}
			}
			
			if (gtr1_CameraMode == TR1_CAMERA_MODE_GOTO_SANDBOX)
			{
				vec_set(gtr1_CamMoveToPos, gtr1_SandboxCameraPos);
				vec_set(gtr1_CamMoveToAng, gtr1_SandboxCameraAngle);

				l_moveTimeLeft = tr1_camera_updateTransition(l_moveTimeLeft);
				if (l_moveTimeLeft <= 0)
				{	// finished transition... goto sandbox
					gtr1_CameraMode = TR1_CAMERA_MODE_PICKUP;
				}
			}
		}
		
		wait(1);
	}
}
	

//********************************************************
// handle keyboard camera controls
//********************************************************
function tr1_Camera_UpdateKeyboard ()
{
	while (1)
	{
		if (gtr1_microgameActive)
		{
			if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
			{
				if (key_alt == on)
				{	// if alt key held, then rotate camera
					gtr1_NewCamMode = TR1_CAMERA_MODE_ROTATE;
				}
		
				if (key_ctrl == on)
 				{	// if shift key held, then strafe camera
					gtr1_NewCamMode = TR1_CAMERA_MODE_STRAFE;
				}
				
				if ((key_w == on) || (key_s == on) || (key_a == on) || (key_d == on))
				{	// using keyboard keys to rotate camera
					gtr1_NewCamMode = TR1_CAMERA_MODE_ROTATE_KEYS;
				}
				
				if ((key_cuu == on) || (key_cud == on) || (key_cul == on) || (key_cur == on) || (key_pgup == on) || (key_pgdn == on))
				{	// using keyboard keys to strafe camera
					gtr1_NewCamMode = TR1_CAMERA_MODE_STRAFE_KEYS;
				}
			}
			else 
			{
				if (   ((gtr1_CameraMode == TR1_CAMERA_MODE_ROTATE) && (key_alt == off))
					 || ((gtr1_CameraMode == TR1_CAMERA_MODE_STRAFE) && (key_ctrl == off))
					 || ((gtr1_CameraMode == TR1_CAMERA_MODE_ROTATE_KEYS) && (key_w == off) && (key_s == off) && (key_a == off) && (key_d == off))
					 || ((gtr1_CameraMode == TR1_CAMERA_MODE_STRAFE_KEYS) && (key_cuu == off) && (key_cud == off) 
					 	  && (key_cul == off) && (key_cur == off) && (key_pgup == off) && (key_pgdn == off)))
				{
					gtr1_NewCamMode = TR1_CAMERA_MODE_PICKUP;
				}
			}
		}
		
		wait(1);
	}
}
