//**********************************************************************
// collision functions
//**********************************************************************


// cursor c_trace collision data
define TR1_NUM_CURSOR_VECTS,	2;

var gtr1_CursorCollideVects[6] =
			  0,   0, -200,
			  0,   0,  200;


//*******************************************************************
// translate a c_trace collision into a haptics surface position 
// and normal
//
// IN: c_trace has just been called, and thus the global target and normal vars are set
//*******************************************************************
function tr1_Collide_CalcHapticsSurface ()
{
	var l_surfacePos[3];
	var l_surfaceNormal[3];

	// set the hit point just above the surface
	vec_set(l_surfaceNormal, normal.x);
	vec_normalize(l_surfaceNormal, 1.0);
				
	vec_set(l_surfacePos, l_surfaceNormal);
	vec_scale(l_surfacePos, 15.0);
	vec_add(l_surfacePos, target.x);
	
	vec_scale(l_surfaceNormal, 50.0);
	
	// get the terrain normal
	vec_set(gtr1_WorldPos, l_surfaceNormal);
				
	if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
	{	// sandbox view
		tr1_Haptics_Sandbox_WorldVecToHaptics();
	}
	else
	{	// driving mode
		tr1_Haptics_Driving_WorldVecToHaptics();
	}

	vec_normalize(gtr1_HapticsPos, 1.0);
	vec_set(l_surfaceNormal, gtr1_HapticsPos);

	// set the haptics floor to the hit point height
	vec_set(gtr1_WorldPos, l_surfacePos);
				
	if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
	{	// sandbox view
		tr1_Haptics_Sandbox_WorldPosToHaptics();
	}
	else
	{	// driving mode
		tr1_Haptics_Driving_WorldPosToHaptics();
	}

	// set the surface in haptics
	tr1_dll_setSurfacePos(gtr1_HapticsPos.x, gtr1_HapticsPos.y, gtr1_HapticsPos.z);
	tr1_dll_setSurfaceNormal(l_surfaceNormal.x, l_surfaceNormal.y, l_surfaceNormal.z);
	// NOTE: should set the stiffness of the surface using tr1_dll_setSurfaceStiffness, but for now just use the default value
}


//****************************************************************
// run cursor to terrain surface collision
//****************************************************************
function tr1_Collide_CursorToTerrain ()
{
	var l_collideStart[3];
	var l_collideEnd[3];
	
	var l_i = 0;

	// get the cursor entity
	tr1_Object_GetEntity(gtr1_CursorID);
	
	// set the start point of the collide rays to the cursor pos
	vec_set(l_collideStart, gtr1_pCurIDEntity.x);

	// create all the needed rays, and set surfaces for any that hit
	while (l_i < (TR1_NUM_CURSOR_VECTS *3))
	{
		vec_set(l_collideEnd, gtr1_CursorCollideVects[l_i]);
		vec_rotate(l_collideEnd, vector(camera.pan, 0, 0));
		vec_add(l_collideEnd, gtr1_pCurIDEntity.x);

		// collide the ray against terrain
		c_trace(l_collideStart, l_collideEnd, (IGNORE_PASSABLE+IGNORE_PASSENTS+IGNORE_MODELS));
			
		if (trace_hit == 1)
		{	// collide ray hit terrain... set a haptics surface to match
			tr1_Collide_CalcHapticsSurface();
		}
		
		l_i += 3;
	}
}


//****************************************************************
// setup haptics for an object
//
// IN:  gtr1_pCurIDEntity holds the entity ptr for the object
//****************************************************************
function tr1_Collide_SetupObjHaptics ()
{
	var l_boxSize[3];
	
	if (gtr1_pCurIDEntity.m_hapticsType == TR1_HAPTICS_TYPE_NONE)
	{	// no haptics
		return;
	}
	
	if (gtr1_pCurIDEntity.m_hapticsType == TR1_HAPTICS_TYPE_SPHERE)
	{	// create a haptics sphere
		vec_set(gtr1_WorldPos, gtr1_pCurIDEntity.x);
				
		if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
		{	// sandbox view
			tr1_Haptics_Sandbox_WorldPosToHaptics();
		}
		else
		{	// driving mode
			tr1_Haptics_Driving_WorldPosToHaptics();
		}

		// set the sphere in haptics
		tr1_dll_setSphere(gtr1_HapticsPos.x, gtr1_HapticsPos.y, gtr1_HapticsPos.z, gtr1_pCurIDEntity.m_hapticsRadius);
		tr1_dll_setSphereStiffness(gtr1_pCurIDEntity.m_hapticsStiffness);
		
		return;
	}

	if (gtr1_pCurIDEntity.m_hapticsType == TR1_HAPTICS_TYPE_BOX)
	{	// create a haptics box
		vec_set(gtr1_WorldPos, gtr1_pCurIDEntity.x);
				
		if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
		{	// sandbox view
			tr1_Haptics_Sandbox_WorldPosToHaptics();
		}
		else
		{	// driving mode
			tr1_Haptics_Driving_WorldPosToHaptics();
		}

		// set the box in haptics
		tr1_dll_setBoxPos(gtr1_HapticsPos.x, gtr1_HapticsPos.y, gtr1_HapticsPos.z);
		
		vec_set(l_boxSize, gtr1_pCurIDEntity.m_hapticsBoxSize);
		tr1_dll_setBoxSize(l_boxSize.x, l_boxSize.y, l_boxSize.z);
			
		vec_set(gtr1_WorldPos, vector(10.0, 0, 0));
		vec_rotate(gtr1_WorldPos, gtr1_pCurIDEntity.pan);

		if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
		{	// sandbox view
			tr1_Haptics_Sandbox_WorldVecToHaptics();
		}
		else
		{	// driving mode
			tr1_Haptics_Driving_WorldVecToHaptics();
		}

		tr1_dll_setBoxHeading(gtr1_HapticsPos.x, gtr1_HapticsPos.y, gtr1_HapticsPos.z);
				
		vec_set(gtr1_WorldPos, vector(0, 0, 10.0));
		vec_rotate(gtr1_WorldPos, gtr1_pCurIDEntity.pan);

		if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
		{	// sandbox view
			tr1_Haptics_Sandbox_WorldVecToHaptics();
		}
		else
		{	// driving mode
			tr1_Haptics_Driving_WorldVecToHaptics();
		}

		tr1_dll_setBoxUp(gtr1_HapticsPos.x, gtr1_HapticsPos.y, gtr1_HapticsPos.z);
		
		tr1_dll_setBoxStiffness(gtr1_pCurIDEntity.m_hapticsStiffness);
		
		return;
	}
}


//****************************************************************
// run haptics collision on the active objects of an input type
//
// IN: type of object
//****************************************************************
function tr1_Collide_Type_Haptics (v_objType)
{
	var l_curID;
	var l_distToObj;
	
	// get the cursor's entity ptr
	tr1_Object_GetEntity(gtr1_CursorID);
	gtr1_pAltIDEntity = gtr1_pCurIDEntity;
	
	// start at top of active list for this type
	l_curID = gtr1_ActiveObjs[v_objType];
	
	// walk through all the objects of this type
	while (l_curID >= 0)
	{
		tr1_Object_GetEntity(l_curID);
		if (gtr1_pCurIDEntity != gtr1_pObjHeld)
		{	// only create haptics effects if not the held object
			l_distToObj = vec_dist(gtr1_pAltIDEntity.x, gtr1_pCurIDEntity.x);

			if (l_distToObj <= gtr1_pCurIDEntity.m_objNearDist)
			{	// near this object
				tr1_Collide_SetupObjHaptics();
			}
		}
		
		// go to next object in list
		l_curID = gtr1_ObjectNext[l_curID];
	}
}


//****************************************************************
// run haptics collision on all haptics capable types of objects
//****************************************************************
function tr1_Collide_Haptics ()
{
	// update temporary forces
	tr1_HapticsForces_Update();

	if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
	{
		// terrain collision
		tr1_Collide_CursorToTerrain();
	
		// object collision
		tr1_Collide_Type_Haptics(TR1_OBJ_TYPE_PHYSICS_OBJ);
		tr1_Collide_Type_Haptics(TR1_OBJ_TYPE_PHYSICS_CAR);
	
		// button collision
		tr1_Collide_Type_Haptics(TR1_OBJ_TYPE_SHRINK_BTN);

		return;
	}

	if ((gtr1_CameraMode == TR1_CAMERA_MODE_DRIVING) && (tr1_dll_isHapticButtonDepressed()))
	{	// button is active, in grow-me type
		tr1_Collide_Type_Haptics(TR1_OBJ_TYPE_GROW_BTN);
		
		return;
	}
}


//***************************************************************
// do object pickup collision against the cursor for an input object type
//
// IN:  object type
//
// RETURNS:  object ID of hit object, -1 if none
//***************************************************************
function tr1_Collide_Type_Pickup (v_objType)
{
	var l_curID;
	
	var l_distToObj;

	// get the cursor's entity ptr
	tr1_Object_GetEntity(gtr1_CursorID);
	gtr1_pAltIDEntity = gtr1_pCurIDEntity;
	
	// start at top of active list for this type
	l_curID = gtr1_ActiveObjs[v_objType];
	
	// walk through all the objects of this type
	while (l_curID >= 0)
	{
		gtr1_pCurIDEntity = ptr_for_handle(gtr1_ObjectHandle[l_curID]);
		
		l_distToObj = vec_dist(gtr1_pAltIDEntity.x, gtr1_pCurIDEntity.x);
		if (l_distToObj <= gtr1_pCurIDEntity.m_objPickupDist)
		{	// hit this object
			return(l_curID);
		}
		
		// go to next object in list
		l_curID = gtr1_ObjectNext[l_curID];
	}
	
	// didn't collide
	return(-1);
}


//********************************************************
// do object pickup collision against the cursor for all object types
//
// RETURNS:  object ID of hit object, -1 if none
//********************************************************
function tr1_Collide_Pickup ()
{
	var l_curType;
	var l_collideID;
	
	if (gtr1_CameraMode == TR1_CAMERA_MODE_DRIVING)
	{	// in driving mode, can't pickup objects
		return(-1);
	}
	
	// in pickup mode... collide against everything
	l_curType = TR1_OBJ_TYPE_CURSOR + 1;	// start with first object type past cursor type
	
	while (l_curType < TR1_OBJ_TYPE_SHRINK_BTN)
	{
		l_collideID = tr1_Collide_Type_Pickup(l_curType);
		if (l_collideID >= 0)
		{	// hit something
			return(l_collideID);
		}
		
		l_curType += 1;
	}
	
	return(-1);
}
	
