//*******************************************************************
// cursor controls
//*******************************************************************


define TR1_CURSOR_SANDBOX_SCALE,	4.0;
define TR1_CURSOR_DRIVING_SCALE,	0.3;

define TR1_CURSOR_PUSH,		10;
define TR1_OBJECT_PUSH,		20;
define TR1_HELDOBJ_PUSH,	0;


//*****************************************************
// Entity Action for the haptics driven cursor
//*****************************************************
action tr1_aCursorBehavior
{
	// start and end points of the terrain height collide ray
	var l_collideStart[3];
	var l_collideEnd[3];
	
	// haptics button values
	var l_btnLast = 0;
	var l_btnPressed = 0;
	var l_btnDown = 0;
	var l_btnUp = 0;
	
	var l_objType;
	
	var l_bClicked = 0;
	
	var l_collideSettings;
	
	my.fat = on;
	my.narrow = on;
	c_setminmax(my);
	
	my.shadow = off;
	my.invisible = on;
	my.passable = on;
	
	my.tilt = 30.0;
	
	my.push = TR1_CURSOR_PUSH;
	
	wait(1);
		
	while(1)
	{
		// set the haptics button flags
		if (tr1_dll_isHapticButtonDepressed())
		{
			l_btnPressed = 1;
		}
		else
		{
			l_btnPressed = 0;
		}
		
		l_btnDown = 0;
		l_btnUp = 0;
		if (l_btnLast == 0)
		{
			if (l_btnPressed != 0)
			{	// button just went down
				l_btnDown = 1;
			}
		}
		else
		{
			if (l_btnPressed == 0)
			{	// button was just released
				l_btnUp = 1;
			}
		}
	
		// reset the haptics surfaces
		tr1_dll_resetObjects();
		
		if ((gtr1_CameraMode == TR1_CAMERA_MODE_DRIVING) && (!l_btnLast))
		{
			tr1_dll_setDrivingForces();
		}

		if (   (gtr1_CameraMode != TR1_CAMERA_MODE_PICKUP) 
			 && ((gtr1_CameraMode != TR1_CAMERA_MODE_DRIVING) || (!l_btnLast)))
		{	// don't do normal cursor processing
				my.invisible = on;
				my.shadow = off;

				// check haptics collisions
				tr1_Collide_Haptics();
		}
		else
		{
			my.invisible = off;
			my.shadow = on;

			// position the cursor from haptics pos, relative to the camera
			if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
			{	// sandbox view
				my.scale_x = TR1_CURSOR_SANDBOX_SCALE;
				my.scale_y = TR1_CURSOR_SANDBOX_SCALE;
				my.scale_z = TR1_CURSOR_SANDBOX_SCALE;

				tr1_Haptics_Sandbox_HapticsToWorld();
			}
			else
			{	// driving mode
				my.scale_x = TR1_CURSOR_DRIVING_SCALE;
				my.scale_y = TR1_CURSOR_DRIVING_SCALE;
				my.scale_z = TR1_CURSOR_DRIVING_SCALE;

				tr1_Haptics_Driving_HapticsToWorld();
			}
			
			vec_set(my.x, gtr1_WorldPos);
		
			vec_set(my.pan, camera.pan);
			my.pan += 180.0;

			vec_set(gtr1_CursorPos, gtr1_HapticsCameraRelPos);

			// check haptics collisions
			tr1_Collide_Haptics();
			
			if (gtr1_microgameActive)
			{
				// now check for picking up/carrying/releasing objects
				if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
				{
					l_bClicked = l_btnDown;
				}
				else
				{
					l_bClicked = l_btnUp;
				}
				
				if (l_bClicked != 0)
				{	// see if we can pick up any objects
					gtr1_HitObjID = tr1_Collide_Pickup();

					if (gtr1_HitObjID >= 0)
					{	// collided with an object
						l_objType = tr1_Object_GetType(gtr1_HitObjID);

						// pick up the object
						gtr1_CursorHoldingObj = 1;
							
						gtr1_pObjHeld = gtr1_pCurIDEntity;
							
						// set the haptics weight
						tr1_dll_setHoldingWeight(gtr1_pCurIDEntity.m_hapticsWeight);

						if (l_objType == TR1_OBJ_TYPE_PHYSICS_CAR)
						{	// its a car... this car becomes the one we will drive
							gtr1_CarID = gtr1_HitObjID;
						}
					}
				}
		
				if ((l_btnUp != 0) && (gtr1_CursorHoldingObj != 0))
				{	// drop the object
					gtr1_CursorHoldingObj = 0;

					// clear the haptics weight
					tr1_dll_setHoldingWeight(-1);
	
					gtr1_pObjHeld = NULL;
				}
			}
		}

		// store the last button press status
		l_btnLast = l_btnPressed;
				
		wait(1);	
	}
}


//*******************************************************
// create the cursor
//
// RETURNS: object ID of the cursor
//*******************************************************
function tr1_Cursor_Create ()
{
	var l_objID;
	var l_objHandle;
	
	if (gtr1_CursorID >= 0)
	{	// already have a cursor!
		return(-1);
	}

	l_objID = tr1_Object_New(TR1_OBJ_TYPE_CURSOR);
	if (l_objID < 0)
	{	// error... no objects left
		return;
	}
	
	// create the cursor
	gtr1_pCurIDEntity = ent_create("hand.mdl", vector(0, 0, 1000), tr1_aCursorBehavior);
	l_objHandle = handle(gtr1_pCurIDEntity);
	
	tr1_Object_Set(l_objID, l_objHandle, TR1_OBJ_TYPE_CURSOR);
	
	return(l_objID);
}
