//***************************************************************
// haptics conversion routines
//***************************************************************


// haptics routine input/output vars
var gtr1_WorldPos[3];
var gtr1_HapticsPos[3];


// temporary haptics forces data
define TR1_NUM_HAPTICS_FORCES,	20;	// number of forces available
define TR1_NUM_HAPTICS_VECTORS,	60;	// number of force vector vars (MUST be TR1_NUM_HAPTICS_FORCES*3)

//--------------------------------------
// temp haptics forces linked list data
//--------------------------------------
var gtr1_HapticsForcesVectors[TR1_NUM_HAPTICS_VECTORS];
var gtr1_HapticsForcesSize[TR1_NUM_HAPTICS_VECTORS];
var gtr1_HapticsForcesTimeLeft[TR1_NUM_HAPTICS_FORCES];
var gtr1_HapticsForcesNext[TR1_NUM_HAPTICS_FORCES];

// unused forces list top
var gtr1_UnusedHapticsForces;

// active forces list top
var gtr1_ActiveHapticsForces;

// input to tr1_HapticsForce_New
var gtr1_HapticsForceVec[3];

// stores current camera relative haptics pos (set from latest HapticsToWorld call)
var gtr1_HapticsCameraRelPos[3];


//*********************************************************************
// convert from haptics coords to world coords (relative to the 
// camera - sandbox view version
//
// OUT: gtr1_WorldPos holds the world pos
//*********************************************************************
function tr1_Haptics_Sandbox_HapticsToWorld ()
{
	var l_offsetX;
	var l_offsetY;
	var l_offsetZ;
	var l_scaleBase;
	var l_scaleZ;
	
	l_offsetX = tr1_dll_getToolPosOffsetX_Sandbox();
	l_offsetY = tr1_dll_getToolPosOffsetY_Sandbox();
	l_offsetZ = tr1_dll_getToolPosOffsetZ_Sandbox();
	l_scaleBase = tr1_dll_getToolMovementScaleBase_Sandbox();
	l_scaleZ = tr1_dll_getToolMovementScaleZ_Sandbox();
	
	// get haptics pos
	// NOTE: the haptics device uses x=left/right, y=up/down, and z=in/out
	//       the 3DGS environment uses x=in/out, y=left/right, and z=up/down
	gtr1_WorldPos.x = -(tr1_dll_getZPos() * l_scaleBase * l_scaleZ) + l_offsetZ;
	gtr1_WorldPos.y = -(tr1_dll_getXPos() * l_scaleBase) + l_offsetX;
	gtr1_WorldPos.z =  (tr1_dll_getYPos() * l_scaleBase) + l_offsetY;

	vec_set(gtr1_HapticsCameraRelPos, gtr1_WorldPos);

	// position the cursor from haptics pos, relative to the camera
	vec_rotate(gtr1_WorldPos, camera.pan);
	vec_add(gtr1_WorldPos, camera.x);
}


//*********************************************************************
// convert a world position from world coords to haptics coords 
// (relative to the camera) - sandbox view version
//
// IN:  gtr1_WorldPos holds the world pos
//
// OUT: gtr1_HapticsPos holds the haptics pos
//*********************************************************************
function tr1_Haptics_Sandbox_WorldPosToHaptics ()
{
	var l_relCamPos[3];
	var l_camHead[3];
	var l_camWing[3];
	var l_camUp[3];
	var l_hapticsPos[3];
	
	var l_offsetX;
	var l_offsetY;
	var l_offsetZ;
	var l_scaleBase;
	var l_scaleZ;
	
	l_offsetX = tr1_dll_getToolPosOffsetX_Sandbox();
	l_offsetY = tr1_dll_getToolPosOffsetY_Sandbox();
	l_offsetZ = tr1_dll_getToolPosOffsetZ_Sandbox();
	l_scaleBase = tr1_dll_getToolMovementScaleBase_Sandbox();
	l_scaleZ = tr1_dll_getToolMovementScaleZ_Sandbox();
	
	// get relative position to camera
	vec_diff(l_relCamPos, gtr1_WorldPos, camera.x);

	// calculate the orientation vectors for the camera
	vec_set(l_camHead, vector(1, 0, 0));
	vec_rotate(l_camHead, camera.pan);
	
	vec_set(l_camWing, vector(0, 1, 0));
	vec_rotate(l_camWing, camera.pan);

	vec_set(l_camUp, vector(0, 0, 1));
	vec_rotate(l_camUp, camera.pan);

	// calculate projection along the camera axis vectors
	l_hapticsPos.x = vec_dot(l_camHead, l_relCamPos);
	l_hapticsPos.y = vec_dot(l_camWing, l_relCamPos);
	l_hapticsPos.z = vec_dot(l_camUp, l_relCamPos);

	// adjust for haptics coord system
	gtr1_HapticsPos.x =  (l_offsetX - l_hapticsPos.y) / l_scaleBase;
	gtr1_HapticsPos.y = -(l_offsetY - l_hapticsPos.z) / l_scaleBase;
	gtr1_HapticsPos.z =  (l_offsetZ - l_hapticsPos.x) / (l_scaleBase * l_scaleZ);
}


//*********************************************************************
// rotate a vector from world coords to haptics coords
// (relative to the camera) - sandbox view
//
// IN:  gtr1_WorldPos holds the world vector
//
// OUT: gtr1_HapticsPos holds the haptics vector
//*********************************************************************
function tr1_Haptics_Sandbox_WorldVecToHaptics ()
{
	var l_camHead[3];
	var l_camWing[3];
	var l_camUp[3];
	var l_hapticsPos[3];

	var l_scaleBase;
	var l_scaleZ;
	
	l_scaleBase = tr1_dll_getToolMovementScaleBase_Sandbox();
	l_scaleZ = tr1_dll_getToolMovementScaleZ_Sandbox();

	// calculate the orientation vectors for the camera
	vec_set(l_camHead, vector(1, 0, 0));
	vec_rotate(l_camHead, camera.pan);
	
	vec_set(l_camWing, vector(0, 1, 0));
	vec_rotate(l_camWing, camera.pan);

	vec_set(l_camUp, vector(0, 0, 1));
	vec_rotate(l_camUp, camera.pan);

	// calculate projection along the camera axis vectors
	l_hapticsPos.x = vec_dot(l_camHead, gtr1_WorldPos);
	l_hapticsPos.y = vec_dot(l_camWing, gtr1_WorldPos);
	l_hapticsPos.z = vec_dot(l_camUp, gtr1_WorldPos);

	// adjust for haptics coord system
	gtr1_HapticsPos.x = -l_hapticsPos.y / l_scaleBase;
	gtr1_HapticsPos.y =  l_hapticsPos.z / l_scaleBase;
	gtr1_HapticsPos.z = -l_hapticsPos.x / (l_scaleBase * l_scaleZ);
}


//*********************************************************************
// convert from haptics coords to world coords (relative to the 
// camera - driving view version
//
// OUT: gtr1_WorldPos holds the world pos
//*********************************************************************
function tr1_Haptics_Driving_HapticsToWorld ()
{
	var l_offsetX;
	var l_offsetY;
	var l_offsetZ;
	var l_scaleBase;
	var l_scaleZ;
	
	l_offsetX = tr1_dll_getToolPosOffsetX_Driving();
	l_offsetY = tr1_dll_getToolPosOffsetY_Driving();
	l_offsetZ = tr1_dll_getToolPosOffsetZ_Driving();
	l_scaleBase = tr1_dll_getToolMovementScaleBase_Driving();
	l_scaleZ = tr1_dll_getToolMovementScaleZ_Driving();
	
	// get haptics pos
	// NOTE: the haptics device uses x=left/right, y=up/down, and z=in/out
	//       the 3DGS environment uses x=in/out, y=left/right, and z=up/down
	gtr1_WorldPos.x = -(tr1_dll_getZPos() * l_scaleBase * l_scaleZ) + l_offsetZ;
	gtr1_WorldPos.y = -(tr1_dll_getXPos() * l_scaleBase) + l_offsetX;
	gtr1_WorldPos.z =  (tr1_dll_getYPos() * l_scaleBase) + l_offsetY;

	vec_set(gtr1_HapticsCameraRelPos, gtr1_WorldPos);

	// position the cursor from haptics pos, relative to the camera
	vec_rotate(gtr1_WorldPos, camera.pan);
	vec_add(gtr1_WorldPos, camera.x);
}


//*********************************************************************
// convert a world position from world coords to haptics coords 
// (relative to the camera) - driving view version
//
// IN:  gtr1_WorldPos holds the world pos
//
// OUT: gtr1_HapticsPos holds the haptics pos
//*********************************************************************
function tr1_Haptics_Driving_WorldPosToHaptics ()
{
	var l_relCamPos[3];
	var l_camHead[3];
	var l_camWing[3];
	var l_camUp[3];
	var l_hapticsPos[3];
	
	var l_offsetX;
	var l_offsetY;
	var l_offsetZ;
	var l_scaleBase;
	var l_scaleZ;
	
	l_offsetX = tr1_dll_getToolPosOffsetX_Driving();
	l_offsetY = tr1_dll_getToolPosOffsetY_Driving();
	l_offsetZ = tr1_dll_getToolPosOffsetZ_Driving();
	l_scaleBase = tr1_dll_getToolMovementScaleBase_Driving();
	l_scaleZ = tr1_dll_getToolMovementScaleZ_Driving();

	// get relative position to camera
	vec_diff(l_relCamPos, gtr1_WorldPos, camera.x);

	// calculate the orientation vectors for the camera
	vec_set(l_camHead, vector(1, 0, 0));
	vec_rotate(l_camHead, camera.pan);
	
	vec_set(l_camWing, vector(0, 1, 0));
	vec_rotate(l_camWing, camera.pan);

	vec_set(l_camUp, vector(0, 0, 1));
	vec_rotate(l_camUp, camera.pan);

	// calculate projection along the camera axis vectors
	l_hapticsPos.x = vec_dot(l_camHead, l_relCamPos);
	l_hapticsPos.y = vec_dot(l_camWing, l_relCamPos);
	l_hapticsPos.z = vec_dot(l_camUp, l_relCamPos);
	
	// adjust for haptics coord system
	gtr1_HapticsPos.x =  (l_offsetX - l_hapticsPos.y) / l_scaleBase;
	gtr1_HapticsPos.y = -(l_offsetY - l_hapticsPos.z) / l_scaleBase;
	gtr1_HapticsPos.z =  (l_offsetZ - l_hapticsPos.x) / (l_scaleBase * l_scaleZ);
}


//*********************************************************************
// rotate a vector from world coords to haptics coords
// (relative to the camera) - driving view
//
// IN:  gtr1_WorldPos holds the world vector
//
// OUT: gtr1_HapticsPos holds the haptics vector
//*********************************************************************
function tr1_Haptics_Driving_WorldVecToHaptics ()
{
	var l_camHead[3];
	var l_camWing[3];
	var l_camUp[3];
	var l_hapticsPos[3];

	var l_scaleBase;
	var l_scaleZ;
	
	l_scaleBase = tr1_dll_getToolMovementScaleBase_Driving();
	l_scaleZ = tr1_dll_getToolMovementScaleZ_Driving();

	// calculate the orientation vectors for the camera
	vec_set(l_camHead, vector(1, 0, 0));
	vec_rotate(l_camHead, camera.pan);
	
	vec_set(l_camWing, vector(0, 1, 0));
	vec_rotate(l_camWing, camera.pan);

	vec_set(l_camUp, vector(0, 0, 1));
	vec_rotate(l_camUp, camera.pan);

	// calculate projection along the camera axis vectors
	l_hapticsPos.x = vec_dot(l_camHead, gtr1_WorldPos);
	l_hapticsPos.y = vec_dot(l_camWing, gtr1_WorldPos);
	l_hapticsPos.z = vec_dot(l_camUp, gtr1_WorldPos);

	// adjust for haptics coord system
	gtr1_HapticsPos.x = -l_hapticsPos.y / l_scaleBase;
	gtr1_HapticsPos.y =  l_hapticsPos.z / l_scaleBase;
	gtr1_HapticsPos.z = -l_hapticsPos.x / (l_scaleBase * l_scaleZ);
}


//********************************************************
// init the haptics forces system
//********************************************************
function tr1_HapticsForces_InitSystem ()
{
	var l_i;
	
	// link up the list of objects
	l_i = 0;
	while (l_i < (TR1_NUM_HAPTICS_FORCES-1))
	{
		gtr1_HapticsForcesTimeLeft[l_i] = -1;
		gtr1_HapticsForcesNext[l_i] = l_i + 1;
		
		l_i += 1;
	}
	
	// end the list
	gtr1_HapticsForcesTimeLeft[l_i] = -1;
	gtr1_HapticsForcesNext = -1;
	
	// set up the unused list to the top of the buffer
	gtr1_UnusedHapticsForces = 1;
	
	// set the active list to empty
	gtr1_ActiveHapticsForces = -1;
}


//********************************************************
// grab a new unused haptics force, adding it to the active list
//
// IN:  gtr1_HapticsForceVec = the initial force vector to apply (will be normalized)
//	     v_size = the force size to apply
//		  v_time = the time this force lasts over
//********************************************************
function tr1_HapticsForce_New (v_size, v_time)
{
	var l_newObjID;

	if (gtr1_UnusedHapticsForces < 0)
	{	// no objects left
		return;
	}
	
	// pop the top object off the unused list
	l_newObjID = gtr1_UnusedHapticsForces;
	gtr1_UnusedHapticsForces = gtr1_HapticsForcesNext[l_newObjID];
	
	// add the object to the top of the active list
	gtr1_HapticsForcesNext[l_newObjID] = gtr1_ActiveHapticsForces;
	gtr1_ActiveHapticsForces = l_newObjID;
	
	// setup the force
	vec_set(gtr1_HapticsForcesVectors[l_newObjID*3], gtr1_HapticsForceVec);
	vec_normalize(gtr1_HapticsForcesVectors[l_newObjID*3], 1.0);
	
	gtr1_HapticsForcesSize[l_newObjID] = v_size;
	gtr1_HapticsForcesTimeLeft[l_newObjID] = v_time;
}


//********************************************************
// delete an active haptics force
//
// IN:  object ID
//********************************************************
function tr1_HapticsForces_Delete (v_objID)
{
	var l_objNext;
	
	var l_curID;
	
	if ((v_objID < 0) || (v_objID >= TR1_NUM_HAPTICS_FORCES))
	{	// id is out of range
		return;
	}
	
	if (v_objID == gtr1_ActiveHapticsForces)
	{	// this object is at the top of the active list
	
		// remove it from its active list
		gtr1_ActiveHapticsForces = gtr1_HapticsForcesNext[v_objID];
		
		// add it to the unused list
		gtr1_HapticsForcesNext[v_objID] = gtr1_UnusedHapticsForces;
		gtr1_UnusedHapticsForces = v_objID;

		gtr1_HapticsForcesTimeLeft[v_objID] = -1;
	}
	else
	{	// we need to search for it
		l_curID = gtr1_ActiveHapticsForces;
		
		while (gtr1_HapticsForcesNext[l_curID] >= 0)
		{
			if (gtr1_HapticsForcesNext[l_curID] == v_objID)
			{	// found it... remove it from the active list
				gtr1_HapticsForcesNext[l_curID] = gtr1_HapticsForcesNext[v_objID];
				
				// add it to the unused list
				gtr1_HapticsForcesNext[v_objID] = gtr1_UnusedHapticsForces;
				gtr1_UnusedHapticsForces = v_objID;
				
				gtr1_HapticsForcesTimeLeft[v_objID] = -1;
				
				break;
			}
			
			l_curID = gtr1_HapticsForcesNext[l_curID];
		}
	}
}


//***********************************************************
// update the active haptics forces
//***********************************************************
function tr1_HapticsForces_Update ()
{
	var l_curID;
	var l_nextID;
	
	l_curID = gtr1_ActiveHapticsForces;
	
	while (l_curID >= 0)
	{
		l_nextID = gtr1_HapticsForcesNext[l_curID];
		
		// update time left
		gtr1_HapticsForcesTimeLeft[l_curID] -= gtr1_frameTicks;
		if (gtr1_HapticsForcesTimeLeft[l_curID] < 0)
		{	// times up... kill this force
			tr1_HapticsForces_Delete(l_curID);
		}
		else
		{	// force is still active
			
			// NOTE: eventually, the force should attenuate over the time it lasts....
			//       Right now, its constant over its lifetime.
			
			// send force to haptics DLL
			vec_set(gtr1_WorldPos, gtr1_HapticsForcesVectors[l_curID*3]);
			if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
			{	// sandbox view
				tr1_Haptics_Sandbox_WorldVecToHaptics();
			}
			else
			{	// driving mode
				tr1_Haptics_Driving_WorldVecToHaptics();
			}
	
			tr1_dll_setForce(gtr1_HapticsPos.x, gtr1_HapticsPos.y, gtr1_HapticsPos.z, gtr1_HapticsForcesSize[l_curID]); 
		}
		
		l_curID = l_nextID;
	}
}
	