//**************************************************************************
// tr1_main.wdl - main script file for the Tonka truck demo
//
// NOTES:
//		All game resource files and game functions are to be named with unique prefix, ('tr1_' for Tonka Truck 1)
//    All global game variables are to be named with g+<unique Prefix>, ('gtr1_' for global variable Tonka Truck 1)
//    All DEFINES are to be named with a capital prefix, ('TR1_' for define Tonka Truck 1)
//    Microgames should use local variables whenever possible.
//    Microgames should dynamically allocate whenever possible.
//			entities (ent_create, ent_remove, ent_purge)
//			panels (pan_create, pan_remove)
//			bmaps (bmap_load, bmap_create, bmap_purge)
//
// developed by: Doug Lanford, (c) 2006 Novint Technologies, Inc
//**************************************************************************


//**************************
// PROGRAM CONFIGURATION
//**************************

var video_mode   = 8; //Game must be 1024x768 resolution
var video_screen = 2; //1; //Game must be Fullscreen for release, but easier to test in windowed

string gtr1_levelWMBString = <tr1_demo.wmb>; //wmb level string is required for loading


//**************************
// BINDS/PATHS/INCLUDES
//**************************

bind <tr1_haptics.ini>;	//Each game must have a unique haptics ini file defined and handled in the Haptic DLL


//**************************
// GAME MODE STATES
//**************************

var gtr1_MicrogameActive = on; //only run while() loop code if the microgame is active, eg. while(gtr1_microgameActive)

// must keep track of total_secs and total_ticks from when the game is started
var gtr1_total_secs = total_secs; //must use total time this microgame has been running, not total_secs which indicates how //long gamestudio has been running
var gtr1_total_ticks = total_ticks; //must use total time this microgame has been running, not total_secs which indicates //how long gamestudio has been running

var gtr1_lastTotalTicks = total_ticks;
var gtr1_frameTicks = 0;


//**************************
// GAME STATUS
//**************************

var gtr1_gameTimePlayed = 0;

define TR1_FONT_SIZE,16;
define TR1_TEXT_MODE,3;
define TR1_TEXT_ALPHA,100;

var gtr1_targetPos[3];


//**************************
// PANELS/IMAGES/FONTS
//**************************

font gtr1_arial_font_large = "Arial", 1, 40; // truetype font


//**************************
// SOUNDS
//**************************
var   gtr1_soundtrackHnd = 0;
sound gtr1_soundtrackWav = <tr1_soundtrack.wav>;
var   gtr1_gameOverHnd = 0;
sound gtr1_gameOverWav = <tr1_dingEnding.wav>;

//These sound variables are required for the NMB Framework communication 
var TR1_SOUND_BACKGROUND = ON;	//background music
var TR1_SOUND_EFFECTS = ON;		//all other sounds and effects
var TR1_SOUND_MUTE = OFF;			//mute all sounds
var TR1_SOUND_VOLUME = 50;			//sound volume, default sounds should sound 'normal' at 50%


//*************************
// TEMP PRINT STRINGS
//*************************
string gtr1_printStr1;
string gtr1_printStr2;
string gtr1_printStr3;
string gtr1_printStr4;


//*******************************
// HAPTIC DLL FXN DECLARATIONS
//*******************************

dllfunction tr1_dll_createHaptics();
dllfunction tr1_dll_destroyHaptics();
dllfunction tr1_dll_syncHaptics(); 
dllfunction tr1_dll_startHaptics();

// get tool position/button state
dllfunction tr1_dll_getXPos();
dllfunction tr1_dll_getYPos();
dllfunction tr1_dll_getZPos();
dllfunction tr1_dll_isHapticButtonDepressed();

// clear haptics objects
dllfunction tr1_dll_resetObjects();

// set surface position/etc.
dllfunction tr1_dll_setSurfacePos(posX, posY, posZ);
dllfunction tr1_dll_setSurfaceNormal(normX, normY, normZ);
dllfunction tr1_dll_setSurfaceStiffness(stiffness);

// create a haptics sphere or box
dllFunction tr1_dll_setSphere(posX, posY, posZ, radius);
dllfunction tr1_dll_setSphereStiffness(stiffness);

dllfunction tr1_dll_setBoxPos(posX, posY, posZ);
dllfunction tr1_dll_setBoxHeading(vecX, vecY, vecZ);
dllfunction tr1_dll_setBoxUp(vecX, vecY, vecZ);
dllfunction tr1_dll_setBoxSize(sizeX, sizeY, sizeZ);
dllfunction tr1_dll_setBoxStiffness(stiffness);

dllfunction tr1_dll_setForce(dirX, dirY, dirZ, forceScale);

dllfunction tr1_dll_setHoldingWeight(weight);

dllfunction tr1_dll_setDrivingForces();

// get haptics.ini tuning values from the DLL
dllfunction tr1_dll_getSurfaceStiffness();
dllfunction tr1_dll_getSphereStiffness();
dllfunction tr1_dll_getBoxStiffness();
dllfunction tr1_dll_getButtonStiffness();
dllfunction tr1_dll_getButtonClickStiffness();

dllfunction tr1_dll_getToolPosOffsetX_Sandbox();
dllfunction tr1_dll_getToolPosOffsetY_Sandbox();
dllfunction tr1_dll_getToolPosOffsetZ_Sandbox();
dllfunction tr1_dll_getToolMovementScaleBase_Sandbox();
dllfunction tr1_dll_getToolMovementScaleZ_Sandbox();

dllfunction tr1_dll_getToolPosOffsetX_Driving();
dllfunction tr1_dll_getToolPosOffsetY_Driving();
dllfunction tr1_dll_getToolPosOffsetZ_Driving();
dllfunction tr1_dll_getToolMovementScaleBase_Driving();
dllfunction tr1_dll_getToolMovementScaleZ_Driving();

dllfunction tr1_dll_getSandboxCameraDistBehind();
dllfunction tr1_dll_getSandboxDistAbove();
dllfunction tr1_dll_getCarCamera1stDistBehind();
dllfunction tr1_dll_getCarCamera1stDistAbove();
dllfunction tr1_dll_getCarCamera3rdDistBehind();
dllfunction tr1_dll_getCarCamera3rdDistAbove();
dllfunction tr1_dll_getCameraMoveTime();

dllfunction tr1_dll_getPhysicsGravity();

dllfunction tr1_dll_getObjHoldingVelocityScale();

dllfunction tr1_dll_getFlowerpotPhysicsMass();
dllfunction tr1_dll_getFlowerpotPhysicsFriction();
dllfunction tr1_dll_getFlowerpotPhysicsDamping();
dllfunction tr1_dll_getFlowerpotScale();
dllfunction tr1_dll_getFlowerpotCollidePickupSize();
dllfunction tr1_dll_getFlowerpotCollideNearSize();
dllfunction tr1_dll_getFlowerpotHapticsWeight();
dllfunction tr1_dll_getFlowerpotHapticsRadius();

dllfunction tr1_dll_getCratePhysicsMass();
dllfunction tr1_dll_getCratePhysicsFriction();
dllfunction tr1_dll_getCratePhysicsDamping();
dllfunction tr1_dll_getCrateScale();
dllfunction tr1_dll_getCrateCollidePickupSize();
dllfunction tr1_dll_getCrateCollideNearSize();
dllfunction tr1_dll_getCrateHapticsWeight();
dllfunction tr1_dll_getCrateHapticsSizeX();
dllfunction tr1_dll_getCrateHapticsSizeY();
dllfunction tr1_dll_getCrateHapticsSizeZ();

dllfunction tr1_dll_getCarFrontAxisDist();
dllfunction tr1_dll_getCarBackAxisDist();
dllfunction tr1_dll_getCarWheelLatDistL();
dllfunction tr1_dll_getCarWheelLatDistR();
dllfunction tr1_dll_getCarWheelHeight();

dllfunction tr1_dll_getCarPhysicsMassWheel();
dllfunction tr1_dll_getCarPhysicsMassChassis();
dllfunction tr1_dll_getCarPhysicsFrictionWheel();
dllfunction tr1_dll_getCarPhysicsFrictionChassis();
dllfunction tr1_dll_getCarPhysicsDampingWheel();
dllfunction tr1_dll_getCarPhysicsDampingChassis();
dllfunction tr1_dll_getCarPhysicsSuspensionERP();
dllfunction tr1_dll_getCarPhysicsSuspensionCFM();
dllfunction tr1_dll_getCarPhysicsMaxTorque();
dllfunction tr1_dll_getCarPhysicsMaxTorqueBrake();
dllfunction tr1_dll_getCarPhysicsSteerScale();
dllfunction tr1_dll_getCarPhysicsMaxSteer();
dllfunction tr1_dll_getCarPhysicsDeadzoneSteer();
dllfunction tr1_dll_getCarPhysicsThrottleScale();
dllfunction tr1_dll_getCarPhysicsMaxThrottle();
dllfunction tr1_dll_getCarPhysicsDeadzoneThrottle();
dllfunction tr1_dll_getCarUprightSpringScale();
dllfunction tr1_dll_getCarHoldingVelocityScale();
dllfunction tr1_dll_getCarCollidePickupSize();
dllfunction tr1_dll_getCarCollideNearSize();
dllfunction tr1_dll_getCarHapticsWeight();
dllfunction tr1_dll_getCarHapticsSizeX();
dllfunction tr1_dll_getCarHapticsSizeY();
dllfunction tr1_dll_getCarHapticsSizeZ();
dllfunction tr1_dll_getChassisImpactThreshold();
dllfunction tr1_dll_getChassisImpactScale();
dllfunction tr1_dll_getWheelImpactThreshold();
dllfunction tr1_dll_getWheelImpactScale();

dllfunction tr1_dll_getShrinkBtnSandboxX();
dllfunction tr1_dll_getShrinkBtnSandboxY();
dllfunction tr1_dll_getShrinkBtnSandboxZ();
dllfunction tr1_dll_getShrinkBtnScale();
dllfunction tr1_dll_getShrinkBtnCollideSize();
dllfunction tr1_dll_getShrinkBtnNearSize();
dllfunction tr1_dll_getShrinkBtnHapticsSize();

dllfunction tr1_dll_getGrowBtnSandboxX();
dllfunction tr1_dll_getGrowBtnSandboxY();
dllfunction tr1_dll_getGrowBtnSandboxZ();
dllfunction tr1_dll_getGrowBtnScale();
dllfunction tr1_dll_getGrowBtnCollideSize();
dllfunction tr1_dll_getGrowBtnNearSize();
dllfunction tr1_dll_getGrowBtnHapticsSize();

dllfunction tr1_dll_getBtnMinScale();
dllfunction tr1_dll_getBtnClickScale();


//*********************************
// cursor vars
//*********************************

var gtr1_CursorID = -1;

var gtr1_CursorHoldingObj = 0;

// object we are holding with the cursor
entity* gtr1_pObjHeld;


//*********************************
// global physics objects
//*********************************

// physics car
var gtr1_CarID = -1;

// object cursor has collided with
var gtr1_HitObjID = -1;


//*********************************
// cursor data
//*********************************

var gtr1_CursorPos[3];	// the cursor position (relative to the camera)


//*********************************
// button objects
//*********************************

var gtr1_ShrinkGrowBtnID = -1;
var gtr1_bButtonPressed = 0;
var gtr1_bButtonDown = 0;
var gtr1_bButtonUp = 0;


//*********************************
// camera modes
//*********************************

define TR1_CAMERA_MODE_NONE,				0;
define TR1_CAMERA_MODE_PICKUP,			1;	// sandbox view - pick up objects with cursor
define TR1_CAMERA_MODE_STRAFE,			2;	// sandbox view - strafe the camera position using the haptics device
define TR1_CAMERA_MODE_STRAFE_KEYS,		3;	// sandbox view - strafe the camera using keyboard keys
define TR1_CAMERA_MODE_ROTATE,			4;	// sandbox view - rotate camera using the haptics device
define TR1_CAMERA_MODE_ROTATE_KEYS,		5; // sandbox view - rotate camera using keyboard keys
define TR1_CAMERA_MODE_DRIVING,			6;	// driving view
define TR1_CAMERA_MODE_GOTO_DRIVING,	7;	// transition from sandbox to driving
define TR1_CAMERA_MODE_GOTO_SANDBOX,	8;	// transition from driving to sandbox

var gtr1_CameraMode = TR1_CAMERA_MODE_PICKUP;
var gtr1_NewCamMode = TR1_CAMERA_MODE_NONE;


//***************************************
// include  other c-script source files
//***************************************

include <tr1_object_system.wdl>;
include <tr1_haptics.wdl>;
include <tr1_collide.wdl>;
include <tr1_camera.wdl>;
include <tr1_cursor.wdl>;
include <tr1_physics_obj.wdl>;
include <tr1_physics_car.wdl>;
include <tr1_shrinkgrow_btn.wdl>;


//***************************************************************************************
// main() - game script entry point
//***************************************************************************************
function main()
{
    tr1_main();
}


//*******************************************************************
// game specific main function called by main()
//*******************************************************************
function tr1_main()
{
	// Always init variables when starting the game, this allows the game to be played multiple times in NMB
	tr1_initVariables();		
	
	// Create haptic DLL communication
	tr1_dll_createHaptics();
	
	// Start communication with Haptic DLL
	tr1_dll_startHaptics();	
   
	// sync haptics device
	tr1_dll_syncHaptics();

	// Load the level
	level_load(gtr1_levelWMBString);	
	
	// Allow level to load
	wait(3);

	// setup the object system
	tr1_Object_InitSystem();

	// init temp haptics forces system
	tr1_HapticsForces_InitSystem();
	
	// Setup camera
	tr1_Camera_LaunchAndUpdate();
	tr1_Camera_UpdateKeyboard();

	// launch the cursor
	gtr1_CursorID = tr1_Cursor_Create();
	
	// launch the Shrink/Grow button
	gtr1_ShrinkGrowBtnID = tr1_ShrinkGrow_Btn_Create();

	// setup the global physics values
	tr1_Physics_GlobalInit();
	tr1_Physics_Car_Init();

	// launch the two cars
	vec_set(gtr1_CarCreatePos, vector(0, 0, 500));
	vec_set(gtr1_CarCreateAng, vector(90, 0, 0));
	gtr1_CarID = tr1_Physics_Car_Create();

	vec_set(gtr1_CarCreatePos, vector(1500, 400, 300));
	vec_set(gtr1_CarCreateAng, vector(0, 0, 0));
	tr1_Physics_Car_Create();

	// launch two flowerpots
	vec_set(gtr1_ObjCreatePos, vector(2000, 0, 300));
	vec_set(gtr1_ObjCreateAng, vector(0, 0, 0));
	tr1_Physics_Flowerpot_Create();

	vec_set(gtr1_ObjCreatePos, vector(0, 3000, 300));
	vec_set(gtr1_ObjCreateAng, vector(0, 0, 0));
	tr1_Physics_Flowerpot_Create();

	// create a stack of crates
	vec_set(gtr1_ObjCreatePos, vector(2000, -800, 70));
	vec_set(gtr1_ObjCreateAng, vector(0, 0, 0));
	tr1_Physics_Crate_Create();
	
	vec_set(gtr1_ObjCreatePos, vector(1830, -800, 70));
	vec_set(gtr1_ObjCreateAng, vector(0, 0, 0));
	tr1_Physics_Crate_Create();

	vec_set(gtr1_ObjCreatePos, vector(1915, -800, 200));
	vec_set(gtr1_ObjCreateAng, vector(0, 0, 0));
	tr1_Physics_Crate_Create();

	// Start the game soundtrack music if background music is enabled
	if ((TR1_SOUND_BACKGROUND == ON) && (TR1_SOUND_MUTE == OFF))
	{
		gtr1_soundtrackHnd = snd_loop(gtr1_soundtrackWav, TR1_SOUND_VOLUME, 0);	
	}	
	
	// Run the main game loop only while the microgame is active
	while (gtr1_microgameActive)
	{	
		// update frame timing
		gtr1_frameTicks = total_ticks - gtr1_lastTotalTicks;
		gtr1_lastTotalTicks = total_ticks;
	
		// sync haptics device
		tr1_dll_syncHaptics();

		// Handle keyboard actions
		tr1_handleKeyboardActions();	
		
		wait(1);
	}
	
	// Do exit tasks
	sleep(4);
	tr1_exitGame();	
}


//****************************************
// GAME FUNCTIONS
//****************************************


//*******************************************************************
// setup the inital values of game variables (called at game start)
//*******************************************************************
function tr1_initVariables ()
{
	gtr1_gameTimePlayed = 0;
	gtr1_microgameActive = on;
	gtr1_soundtrackHnd = 0;
	gtr1_gameOverHnd = 0;
	gtr1_total_secs = total_secs;
	gtr1_total_ticks = total_ticks;	
	
	gtr1_lastTotalTicks = total_ticks;
	gtr1_frameTicks = 0;
	
	TR1_SOUND_BACKGROUND = on;
	TR1_SOUND_EFFECTS = on;
	TR1_SOUND_MUTE = off;
	TR1_SOUND_VOLUME = 50;	
}


//******************************************************************
// return the relative time (in ticks) since the microgame started
//******************************************************************
function tr1_getGameTicks ()
{
	return (total_ticks - gtr1_total_ticks);
}


//*******************************************************************
// return the relative time (in seconds) since the microgame started
//*******************************************************************
function tr1_getGameSecs ()
{
	return (total_secs - gtr1_total_secs);
}


//****************************************************************
// Check for all relevant key inputs and handle them accordingly
//****************************************************************
function tr1_handleKeyboardActions ()
{
	//An ESC press causes an early termination of the microgame
	if(key_esc == on)
	{
		// Exit the game cleanly
		tr1_exitGame();
	}	
}


//******************************************************************
// This function handles the shutdown of the microgame (when ESC is 
// pressed)
//******************************************************************
function tr1_exitGame ()
{
	// Set flag to shutdown the game
	gtr1_microgameActive = off;
	
	// Destroy haptics
	tr1_dll_destroyHaptics();
	
	wait(10);	
	
	// Stop all sounds
	tr1_stopAllSounds();

	exit;	
}


//*********************************************************************
// Play a sound effect
//
// IN: a_effect = handle to the sound effect
//*********************************************************************
function tr1_playSoundEffect (a_effect)
{
	// Ensure that sound effects are enabled and the game is not muted
	if ((TR1_SOUND_EFFECTS == ON) && (TR1_SOUND_MUTE == OFF))
	{
		snd_play(a_effect, TR1_SOUND_VOLUME,0);
	}
}


//*********************************************************************
// stop all sounds
//*********************************************************************
function tr1_stopAllSounds()
{
	if (gtr1_soundtrackHnd)
	{
		snd_stop(gtr1_soundtrackHnd);
	}
	
	if (gtr1_gameOverHnd)
	{
		snd_stop(gtr1_gameOverHnd);
	}

}  

