//****************************************************************
//* car physics
//****************************************************************


// Car Physics constants
var TR1_CAR_PHYSICS_DAMPING_WHEEL;		// angular damping on the car wheels
var TR1_CAR_PHYSICS_DAMPING_CHASSIS;	// damping on the car chassis

var TR1_CAR_PHYSICS_SUSPENSION_ERP;		// how quickly shocks are reduced to correct position (100,000 max, 0: never)
var TR1_CAR_PHYSICS_SUSPENSION_CFM;		// how far shocks can overshoot (0: never, 100,000 max)

var TR1_CAR_PHYSICS_MAX_TORQUE;			// maximum torque applied to wheels
var TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE;	// maximum torque when braking

var TR1_CAR_PHYSICS_STEER_SCALE;			// steering scale factor
var TR1_CAR_PHYSICS_MAX_STEER;			// max steer angle
var TR1_CAR_PHYSICS_DEADZONE_STEER;		// dead-zone at center of haptics control

var TR1_CAR_PHYSICS_THROTTLE_SCALE;		// throttle scale factor
var TR1_CAR_PHYSICS_MAX_THROTTLE;		// max throttle
var TR1_CAR_PHYSICS_DEADZONE_THROTTLE;	// dead-zone at center of haptics control, throttle

var TR1_CAR_UPRIGHT_SPRING_SCALE;		// scale factor for the spring that keeps the car upright

var TR1_CAR_HOLDING_VELOCITY_SCALE;		// velocity scale to use when holding the car

var TR1_CHASSIS_IMPACT_THRESHOLD;		// speed threshold that causes the chassis to generate a haptics impact force
var TR1_CHASSIS_IMPACT_SCALE;				// scale to apply to chassis impact haptics forces

var TR1_WHEEL_IMPACT_THRESHOLD;			// speed threshold that causes a wheel to generate a haptics impact force
var TR1_WHEEL_IMPACT_SCALE;				// scale to apply to wheel impact haptics forces


// car local vars, stored in the chassis entity's skills
// NOTE: skill1-skill32 are reserved for the object system
define m_handle_wheelFL,		skill33;		// wheel entity ptrs
define m_handle_wheelFR,		skill34;
define m_handle_wheelBL,		skill35;
define m_handle_wheelBR,		skill36;
define m_wheelConstraintFL,	skill37;		// wheel constraints
define m_wheelConstraintFR,	skill38;
define m_wheelConstraintBL,	skill39;
define m_wheelConstraintBR,	skill40;
define m_bOnGround,				skill41;		// flag for whether the car is on the ground or not
define m_chassisVelocity,		skill42;		// 3 vars, holding last velocity of the chassis

// wheel local vars
define m_handle_chassis,		skill33;		// handle to the chassis the wheel is attached to
define m_bWheelOnGround,		skill34;		// is this wheel on the ground?
define m_bWheelPrevOnGround,	skill35;		// was the wheel on the ground last frame?
define m_wheelVelocity,			skill36;		// 3 vars, holding last velocity of this wheel


var gtr1_CarVecUp[3] = 0.0, 0.0, 1.0; 		// up axis, usually +Z
var gtr1_CarVecRight[3] = 0.0, -1.0, 0.0;	// chassis right axis

// inputs to Physics_Car_Create (also accessed by car action behavior when it inits the car)
var gtr1_CarCreatePos[3];
var gtr1_CarCreateAng[3];

// other vars
entity* gtr1_pCarObj;
entity* gtr1_pCurWheel;

var gtr1_Group;


//***************************************************************
// setup the car constants from the .ini file
//***************************************************************
function tr1_Physics_Car_Init ()
{
	TR1_CAR_PHYSICS_DAMPING_WHEEL = tr1_dll_getCarPhysicsDampingWheel();
	TR1_CAR_PHYSICS_DAMPING_CHASSIS = tr1_dll_getCarPhysicsDampingChassis();

	TR1_CAR_PHYSICS_SUSPENSION_ERP = tr1_dll_getCarPhysicsSuspensionERP();
	TR1_CAR_PHYSICS_SUSPENSION_CFM = tr1_dll_getCarPhysicsSuspensionCFM();

	TR1_CAR_PHYSICS_MAX_TORQUE = tr1_dll_getCarPhysicsMaxTorque();
	TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE = tr1_dll_getCarPhysicsMaxTorqueBrake();

	TR1_CAR_PHYSICS_STEER_SCALE = tr1_dll_getCarPhysicsSteerScale();
	TR1_CAR_PHYSICS_MAX_STEER = tr1_dll_getCarPhysicsMaxSteer();
	TR1_CAR_PHYSICS_DEADZONE_STEER = tr1_dll_getCarPhysicsDeadzoneSteer();

	TR1_CAR_PHYSICS_THROTTLE_SCALE = tr1_dll_getCarPhysicsThrottleScale();
	TR1_CAR_PHYSICS_MAX_THROTTLE = tr1_dll_getCarPhysicsMaxThrottle();
	TR1_CAR_PHYSICS_DEADZONE_THROTTLE = tr1_dll_getCarPhysicsDeadzoneThrottle();

	TR1_CAR_UPRIGHT_SPRING_SCALE = tr1_dll_getCarUprightSpringScale();
	
	TR1_CAR_HOLDING_VELOCITY_SCALE = tr1_dll_getCarHoldingVelocityScale();
	
	TR1_CHASSIS_IMPACT_THRESHOLD = tr1_dll_getChassisImpactThreshold();
	TR1_CHASSIS_IMPACT_SCALE = tr1_dll_getChassisImpactScale();
	TR1_WHEEL_IMPACT_THRESHOLD = tr1_dll_getWheelImpactThreshold();
	TR1_WHEEL_IMPACT_SCALE = tr1_dll_getWheelImpactScale();
}


//**************************************************************
// impact event for the car chassis
//**************************************************************
function tr1_Physics_Car_Event_ChassisImpact ()
{
	var l_speed;
	var l_normal[3];
	
	// get the projection of the chassis's velocity along the hit normal
	vec_set(l_normal, normal.x);
	vec_normalize(l_normal, 1.0);
	l_speed = vec_dot(l_normal, my.m_chassisVelocity);
		
	if (l_speed > TR1_CHASSIS_IMPACT_THRESHOLD)
	{	// above the threshold... generate a haptics force
		l_speed -= TR1_CHASSIS_IMPACT_THRESHOLD;
		l_speed *= TR1_CHASSIS_IMPACT_SCALE;
		
		vec_inverse(l_normal);

		vec_set(gtr1_HapticsForceVec, l_normal);
		tr1_HapticsForce_New(l_speed, 1);
	}
}


//***************************************************************
// setup the car chassis
//
// IN: entity to setup
//     gtr1_Group = the group to use
//***************************************************************
function tr1_Physics_Car_ChassisInit (v_pObj)
{
	var l_prevObj;
	
	l_prevObj = handle(my);
	my = v_pObj;
	
	my.passable = on;
	
	vec_set(gtr1_CarVecRight, vector(0, -1, 0));
	vec_rotate(gtr1_CarVecRight, my.pan);   // rotate right vector to correspond to car orientation
	
	// setup physics body for the chassis
	phent_settype(my, PH_RIGID, PH_BOX);
	phent_setmass(my, tr1_dll_getCarPhysicsMassChassis(), PH_BOX);
	phent_setgroup(my, gtr1_Group); 		// all car parts in group2 ->no Collisions
	phent_setfriction(my, tr1_dll_getCarPhysicsFrictionChassis());
	phent_setdamping(my, TR1_CAR_PHYSICS_DAMPING_CHASSIS, TR1_CAR_PHYSICS_DAMPING_CHASSIS);
	phent_setelasticity(my, 10, 100); // little bouncing

	// setup the event handler for the chassis
	my.event = tr1_Physics_Car_Event_ChassisImpact;
	my.enable_friction = on;
	my.enable_block = on;
	my.enable_entity = on;
	
	my.passable = off;
	
	my = ptr_for_handle(l_prevObj);
}


//**************************************************************
// impact event for the car wheels (when it is touching the ground)
//**************************************************************
function tr1_Physics_Car_Event_WheelImpact ()
{
	var l_speed;
	var l_normal[3];
	var l_recoilVec[3];
	
	// this wheel is hitting the ground
	gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_chassis);
	
	gtr1_pCurIDEntity.m_bOnGround = 1;

	if ((my.m_bWheelPrevOnGround == 0) && (gtr1_CameraMode == TR1_CAMERA_MODE_DRIVING))
	{	// was not on ground previously (and in driving mode)

		// get the projection of the wheel's velocity along the hit normal
		vec_set(l_normal, normal.x);
		vec_normalize(l_normal, 1.0);
		l_speed = abs(vec_dot(l_normal, my.m_wheelVelocity));
		
		if (l_speed > TR1_WHEEL_IMPACT_THRESHOLD)
		{	// above the threshold... generate a haptics force
			l_speed -= TR1_WHEEL_IMPACT_THRESHOLD;
			l_speed *= TR1_WHEEL_IMPACT_SCALE;

			// get the vector to apply the force from
			vec_diff(l_recoilVec, gtr1_pCurIDEntity.x, my.x);
			vec_normalize(l_recoilVec, 1.0);
		
			vec_add(l_recoilVec, vector(0, 0, 1));
			vec_normalize(l_recoilVec, 1.0);
		
			vec_set(gtr1_HapticsForceVec, l_recoilVec);
			tr1_HapticsForce_New(l_speed, 1);
		}
	}
	
	my.m_bWheelOnGround = 1;
}


//***************************************************************
// setup a wheel
//
// IN: pObj = entity to setup
//     gtr1_Group = the group to use
//
// RETURNS: wheel contraint for the wheel
//***************************************************************
function tr1_Physics_Car_WheelInit (v_pObj)
{
	var l_obj_constraint;
	
	var l_prevObj;

	l_prevObj = handle(my);
	my = v_pObj;
	
	my.passable = on;
	
	// setup the physics object for this wheel
	phent_settype(my, PH_RIGID, PH_SPHERE);
	phent_setmass(my, tr1_dll_getCarPhysicsMassWheel(), PH_SPHERE);
	phent_setgroup(my, gtr1_Group);
	phent_setfriction(my, tr1_dll_getCarPhysicsFrictionWheel());
	phent_setdamping(my, 15, TR1_CAR_PHYSICS_DAMPING_WHEEL);
	phent_setelasticity(my, 0, 100); // bounciness

	// make wheel constraint pointing up and towards center
	l_obj_constraint = phcon_add(PH_WHEEL, gtr1_pCarObj, my);
	phcon_setparams1(l_obj_constraint, my.x, gtr1_CarVecUp, gtr1_CarVecRight);
	phcon_setparams2(l_obj_constraint, vector(0, 0, 0), nullvector, vector(TR1_CAR_PHYSICS_SUSPENSION_ERP, TR1_CAR_PHYSICS_SUSPENSION_CFM, 0));
	
	// setup the event handler for the wheel
	my.event = tr1_Physics_Car_Event_WheelImpact;
	my.enable_friction = on;
	my.enable_block = on;
	my.enable_entity = on;
		
	my.passable = off;
	
	my.m_handle_chassis = handle(gtr1_pCarObj);
	
	my = ptr_for_handle(l_prevObj);

	return(l_obj_constraint);
}


//**************************************************************
// update the roll/tilt springs that keep the car upright
//
// IN: my is set to the car
//**************************************************************
function tr1_Physics_Car_UpdateRollSprings ()
{
	var l_worldUpVec[3] = 0.0, 0.0, 1.0;
	var l_carUpVec[3];
	var l_rotDelta[3];
	var l_rotPos[3];
	
	var l_length;
	
	// rotate the up vec to match the car's orientation
	vec_set(l_carUpVec, l_worldUpVec);
	vec_rotate(l_carUpVec, my.pan);
	
	// get the difference between the car and up vecs
	vec_diff(l_rotDelta, l_worldUpVec, l_carUpVec);
	
	// scale the forces
	l_length = vec_length(l_rotDelta);
	vec_scale(l_rotDelta, (l_length * l_length * TR1_CAR_UPRIGHT_SPRING_SCALE));
	l_rotDelta.z = 0;
	
	phent_addtorquelocal(my, l_rotDelta);
}


//**************************************************************
// action for initing and running the car chassis
//**************************************************************
action tr1_aCar_Update ()
{
 	var l_wheelPos[3];
 	
	// driving vars
	var l_steerAmount;
	var l_throttle;
	
	var l_headingVec[3];
	var l_velocityVec[3];
	var l_speed;
	var l_velocityAngle;
	var l_bApplyBrakes;
	
	var l_bPhysicsEnabled = 0;

	var l_vecToCursor[3];
	var l_holdPos[3];

	wait(1);	// wait for object system values to be set

	my.fat = on; 
	my.narrow = on; 
	c_setminmax(my);

	my.shadow = on;

	gtr1_pCarObj = my;

	// create the car parts
	gtr1_Group = my.m_objID;
	
	// init the chassis
	tr1_Physics_Car_ChassisInit(my);
	
	// create the FL wheel
	vec_set(l_wheelPos, vector(tr1_dll_getCarFrontAxisDist(), tr1_dll_getCarWheelLatDistL(), tr1_dll_getCarWheelHeight()));
	vec_rotate(l_wheelPos, gtr1_pCarObj.pan);
	vec_add(l_wheelPos, gtr1_pCarObj.x);
	
	gtr1_pCurWheel = ent_create("buggy_rad1.mdl", l_wheelPos, NULL);
	vec_set(gtr1_pCurWheel.pan, gtr1_pCarObj.pan);
	
	gtr1_pCarObj.m_handle_wheelFL = handle(gtr1_pCurWheel);

	gtr1_pCarObj.m_wheelConstraintFL = tr1_Physics_Car_WheelInit(gtr1_pCurWheel);
	
	// create the FR wheel
	vec_set(l_wheelPos, vector(tr1_dll_getCarFrontAxisDist(), tr1_dll_getCarWheelLatDistR(), tr1_dll_getCarWheelHeight()));
	vec_rotate(l_wheelPos, gtr1_pCarObj.pan);
	vec_add(l_wheelPos, gtr1_pCarObj.x);
	
	gtr1_pCurWheel = ent_create("buggy_rad1.mdl", l_wheelPos, NULL);
	vec_set(gtr1_pCurWheel.pan, gtr1_pCarObj.pan);
	
	gtr1_pCarObj.m_handle_wheelFR = handle(gtr1_pCurWheel);

	gtr1_pCarObj.m_wheelConstraintFR = tr1_Physics_Car_WheelInit(gtr1_pCurWheel);
	
	// create the BL wheel
	vec_set(l_wheelPos, vector(tr1_dll_getCarBackAxisDist(), tr1_dll_getCarWheelLatDistL(), tr1_dll_getCarWheelHeight()));
	vec_rotate(l_wheelPos, gtr1_pCarObj.pan);
	vec_add(l_wheelPos, gtr1_pCarObj.x);
	
	gtr1_pCurWheel = ent_create("buggy_rad1.mdl", l_wheelPos, NULL);
	vec_set(gtr1_pCurWheel.pan, gtr1_pCarObj.pan);
	
	gtr1_pCarObj.m_handle_wheelBL = handle(gtr1_pCurWheel);

	gtr1_pCarObj.m_wheelConstraintBL = tr1_Physics_Car_WheelInit(gtr1_pCurWheel);
	
	// create the BR wheel
	vec_set(l_wheelPos, vector(tr1_dll_getCarBackAxisDist(), tr1_dll_getCarWheelLatDistR(), tr1_dll_getCarWheelHeight()));
	vec_rotate(l_wheelPos, gtr1_pCarObj.pan);
	vec_add(l_wheelPos, gtr1_pCarObj.x);
	
	gtr1_pCurWheel = ent_create("buggy_rad1.mdl", l_wheelPos, NULL);
	vec_set(gtr1_pCurWheel.pan, gtr1_pCarObj.pan);
	
	gtr1_pCarObj.m_handle_wheelBR = handle(gtr1_pCurWheel);

	gtr1_pCarObj.m_wheelConstraintBR = tr1_Physics_Car_WheelInit(gtr1_pCurWheel);

	// startup the physics for the car and wheels
	ph_selectgroup(2);

	my = gtr1_pCarObj;

	my.m_bOnGround = 0;

	// update loop of the car
	while (1)
	{
		if (my != gtr1_pObjHeld)
		{	// I'm not being held by the cursor
			if (l_bPhysicsEnabled == 0)
			{	// 
				// set for dropping
				phent_setdamping(my, TR1_CAR_PHYSICS_DAMPING_CHASSIS, TR1_CAR_PHYSICS_DAMPING_CHASSIS);
				phent_setdamping(ptr_for_handle(my.m_handle_wheelFL), TR1_CAR_PHYSICS_DAMPING_WHEEL, TR1_CAR_PHYSICS_DAMPING_WHEEL);
				phent_setdamping(ptr_for_handle(my.m_handle_wheelFR), TR1_CAR_PHYSICS_DAMPING_WHEEL, TR1_CAR_PHYSICS_DAMPING_WHEEL);
				phent_setdamping(ptr_for_handle(my.m_handle_wheelBL), TR1_CAR_PHYSICS_DAMPING_WHEEL, TR1_CAR_PHYSICS_DAMPING_WHEEL);
				phent_setdamping(ptr_for_handle(my.m_handle_wheelBR), TR1_CAR_PHYSICS_DAMPING_WHEEL, TR1_CAR_PHYSICS_DAMPING_WHEEL);
				
				l_bPhysicsEnabled = 1;
				
				// set my push values to be higher than the cursor
				my.push = TR1_OBJECT_PUSH;
				gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelFL);
				gtr1_pCurIDEntity.push = TR1_OBJECT_PUSH;
				gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelFR);
				gtr1_pCurIDEntity.push = TR1_OBJECT_PUSH;
				gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelBL);
				gtr1_pCurIDEntity.push = TR1_OBJECT_PUSH;
				gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelBR);
				gtr1_pCurIDEntity.push = TR1_OBJECT_PUSH;
			}
			
			if ((my.m_objID != gtr1_CarID) || (gtr1_CameraMode != TR1_CAMERA_MODE_DRIVING) || tr1_dll_isHapticButtonDepressed())
			{	// not driving... apply brakes
				phcon_setmotor(my.m_wheelConstraintFL, nullvector, vector(0, TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE, 0), nullvector);
				phcon_setmotor(my.m_wheelConstraintFR, nullvector, vector(0, TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE, 0), nullvector);
				phcon_setmotor(my.m_wheelConstraintBL, nullvector, vector(0, TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE, 0), nullvector);
				phcon_setmotor(my.m_wheelConstraintBR, nullvector, vector(0, TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE, 0), nullvector);
			}
			else
			{	// driving mode
		
				l_steerAmount = tr1_dll_getXPos();
				
		 		if (abs(l_steerAmount) > TR1_CAR_PHYSICS_DEADZONE_STEER)
		 		{	// adjust vehicle steering
		 			if (l_steerAmount < 0)
		 			{
		 				l_steerAmount += TR1_CAR_PHYSICS_DEADZONE_STEER;
	 				}
	 				else
	 				{
						l_steerAmount -= TR1_CAR_PHYSICS_DEADZONE_STEER;
					}
					
					l_steerAmount *= TR1_CAR_PHYSICS_STEER_SCALE;
					l_steerAmount = clamp(l_steerAmount, -TR1_CAR_PHYSICS_MAX_STEER, TR1_CAR_PHYSICS_MAX_STEER);
		
					// adjust steering angle for the front tires
					phcon_setparams2(my.m_wheelConstraintFL, vector(l_steerAmount-2, l_steerAmount+2, 0), nullvector, 
										  vector(TR1_CAR_PHYSICS_SUSPENSION_ERP, TR1_CAR_PHYSICS_SUSPENSION_CFM, 0));
					phcon_setparams2(my.m_wheelConstraintFR, vector(l_steerAmount-2, l_steerAmount+2, 0), nullvector, 
										  vector(TR1_CAR_PHYSICS_SUSPENSION_ERP, TR1_CAR_PHYSICS_SUSPENSION_CFM, 0));
				}
				
			 	l_throttle = -tr1_dll_getZPos();
		 		
		 		if (abs(l_throttle) <= TR1_CAR_PHYSICS_DEADZONE_THROTTLE)
		 		{	// in the dead-zone
		 			l_bApplyBrakes = 1;
	 			}
	 			else
		 		{	// adjust vehicle throttle
		 			if (l_throttle < 0)
		 			{
		 				l_throttle += TR1_CAR_PHYSICS_DEADZONE_THROTTLE;
	 				}
	 				else
	 				{
						l_throttle -= TR1_CAR_PHYSICS_DEADZONE_THROTTLE;
					}

 					l_throttle *= TR1_CAR_PHYSICS_THROTTLE_SCALE;
					l_throttle = clamp(l_throttle, -TR1_CAR_PHYSICS_MAX_THROTTLE, TR1_CAR_PHYSICS_MAX_THROTTLE);

					// check if we are reversing direction
					vec_for_angle(l_headingVec, my.pan);
					phent_getvelocity(my, l_velocityVec, vector(0, 0, 0));
					l_speed = vec_length(l_velocityVec);

					if (l_speed < 5)
					{	// almost stopped... just apply throttle
						l_bApplyBrakes = 0;
					}
					else
					{	// check direction of throttle and velocity	
						l_velocityAngle = vec_dot(l_headingVec, l_velocityVec);
						if (l_velocityAngle > 0)
						{	// velocity is forward
							if (l_throttle > 0)
							{	// throttle is also forward... set wheel motors
								l_bApplyBrakes = 0;
							}
							else
							{	// throttle is backwards... apply brakes
								l_bApplyBrakes = 1;
							}
						}
						else
						{	// velocity is backward
							if (l_throttle > 0)
							{	// throttle is forward... apply brakes
								l_bApplyBrakes = 1;
							}
							else
							{	// throttle is also backwards... set wheel motors
								l_bApplyBrakes = 0;
							}
						}
					}
				}
		
				if (l_bApplyBrakes == 0)
				{	// apply the throttle to the wheel motors
					phcon_setmotor(my.m_wheelConstraintFL, nullvector, vector(l_throttle, TR1_CAR_PHYSICS_MAX_TORQUE, 0), nullvector);
					phcon_setmotor(my.m_wheelConstraintFR, nullvector, vector(l_throttle, TR1_CAR_PHYSICS_MAX_TORQUE, 0), nullvector);
					phcon_setmotor(my.m_wheelConstraintBL, nullvector, vector(l_throttle, TR1_CAR_PHYSICS_MAX_TORQUE, 0), nullvector);
					phcon_setmotor(my.m_wheelConstraintBR, nullvector, vector(l_throttle, TR1_CAR_PHYSICS_MAX_TORQUE, 0), nullvector);
				}
				else
				{	// apply brakes
					phcon_setmotor(my.m_wheelConstraintFL, nullvector, vector(0, TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE, 0), nullvector);
					phcon_setmotor(my.m_wheelConstraintFR, nullvector, vector(0, TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE, 0), nullvector);
					phcon_setmotor(my.m_wheelConstraintBL, nullvector, vector(0, TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE, 0), nullvector);
					phcon_setmotor(my.m_wheelConstraintBR, nullvector, vector(0, TR1_CAR_PHYSICS_MAX_TORQUE_BRAKE, 0), nullvector);
				}
			}
		}
		else
		{	// currently being held by the cursor
			if (l_bPhysicsEnabled != 0)
			{	
				phent_setdamping(my, 100, 100);
				phent_setdamping(ptr_for_handle(my.m_handle_wheelFL), 100, 0);
				phent_setdamping(ptr_for_handle(my.m_handle_wheelFR), 100, 0);
				phent_setdamping(ptr_for_handle(my.m_handle_wheelBL), 100, 0);
				phent_setdamping(ptr_for_handle(my.m_handle_wheelBR), 100, 0);
				
				l_bPhysicsEnabled = 0;

				// set my push values to be lower than the cursor
				my.push = TR1_HELDOBJ_PUSH;
				gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelFL);
				gtr1_pCurIDEntity.push = TR1_HELDOBJ_PUSH;
				gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelFR);
				gtr1_pCurIDEntity.push = TR1_HELDOBJ_PUSH;
				gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelBL);
				gtr1_pCurIDEntity.push = TR1_HELDOBJ_PUSH;
				gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelBR);
				gtr1_pCurIDEntity.push = TR1_HELDOBJ_PUSH;
			}
			
			// halt previous velocity
			phent_clearvelocity(my);
			
			if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
			{	// use the cursor's current position
				tr1_Object_GetEntity(gtr1_CursorID);
				vec_set(l_holdPos, gtr1_pCurIDEntity.x);
			}
			else
			{	// use the cursor's last position, relative to camera
				vec_set(l_holdPos, gtr1_CursorPos);
				vec_rotate(l_holdPos, camera.pan);
				vec_add(l_holdPos, camera.x);
			}
					
			// get velocity to move by
			vec_diff(l_vecToCursor, l_holdPos, my.x);
			vec_scale(l_vecToCursor, TR1_CAR_HOLDING_VELOCITY_SCALE);
			
			phent_addvelcentral(my, l_vecToCursor);
		}

		// update tilt/roll springs
		tr1_Physics_Car_UpdateRollSprings(my.m_bOnGround);

		// reset the on-ground flag and store chassis velocity
		my.m_bOnGround = 0;
		phent_getvelocity(my, my.m_chassisVelocity, nullvector);
		
		// reset the wheel on-ground flags and store each wheel's velocity
		gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelFL);
		gtr1_pCurIDEntity.m_bWheelPrevOnGround = gtr1_pCurIDEntity.m_bWheelOnGround;
		gtr1_pCurIDEntity.m_bWheelOnGround = 0;
		phent_getvelocity(gtr1_pCurIDEntity, gtr1_pCurIDEntity.m_wheelVelocity, nullvector);
		
		gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelFR);
		gtr1_pCurIDEntity.m_bWheelPrevOnGround = gtr1_pCurIDEntity.m_bWheelOnGround;
		gtr1_pCurIDEntity.m_bWheelOnGround = 0;
		phent_getvelocity(gtr1_pCurIDEntity, gtr1_pCurIDEntity.m_wheelVelocity, nullvector);

		gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelBL);
		gtr1_pCurIDEntity.m_bWheelPrevOnGround = gtr1_pCurIDEntity.m_bWheelOnGround;
		gtr1_pCurIDEntity.m_bWheelOnGround = 0;
		phent_getvelocity(gtr1_pCurIDEntity, gtr1_pCurIDEntity.m_wheelVelocity, nullvector);

		gtr1_pCurIDEntity = ptr_for_handle(my.m_handle_wheelBR);
		gtr1_pCurIDEntity.m_bWheelPrevOnGround = gtr1_pCurIDEntity.m_bWheelOnGround;
		gtr1_pCurIDEntity.m_bWheelOnGround = 0;
		phent_getvelocity(gtr1_pCurIDEntity, gtr1_pCurIDEntity.m_wheelVelocity, nullvector);

		wait(1);
	}

}


//*******************************************************
// create a car
//
// IN:  gCarCreatePos = vector holding the position to launch the car at
//      gCarCreateAng = the rotation of the car
//
// RETURNS: object ID of the car
//*******************************************************
function tr1_Physics_Car_Create ()
{
	var l_objID;
	var l_objHandle;

	l_objID = tr1_Object_New(TR1_OBJ_TYPE_PHYSICS_CAR);
	if (l_objID < 0)
	{	// error... no objects left
		return(-1);
	}
	
	// create the car chassis (the tr1_aCar_Update routine will automatically create the wheels/etc.)
	gtr1_pCurIDEntity = ent_create("buggy.mdl", gtr1_CarCreatePos, tr1_aCar_Update);
	l_objHandle = handle(gtr1_pCurIDEntity);
	
	// set its orientation
	vec_set(gtr1_pCurIDEntity.pan, gtr1_CarCreateAng);

	// set the object settings
	tr1_Object_Set(l_objID, l_objHandle, TR1_OBJ_TYPE_PHYSICS_CAR);
	
	// setup collision/haptics for the car
	tr1_Object_SetCollideInfo(l_objID, tr1_dll_getCarCollidePickupSize(), tr1_dll_getCarCollideNearSize(), tr1_dll_getCarHapticsWeight());
	
	vec_set(gtr1_BoxSize, vector(tr1_dll_getCarHapticsSizeX(), tr1_dll_getCarHapticsSizeY(), tr1_dll_getCarHapticsSizeZ()));
	tr1_Object_SetHapticsBox(l_objID, tr1_dll_getBoxStiffness());
	
	return(l_objID);
}


/* NOTE: this will not work anymore, since I'm storing the 
			values local to the chassis entity.  If I later 
			need to be able to remove a car from the game, I'll 
			need to rewrite this.
//*******************************************************
// remove the physics car and all its parts
//*******************************************************
function tr1_Physics_Car_Remove ()
{
	phcon_remove(gtr1_WheelConstraintFL);
	phcon_remove(gtr1_WheelConstraintFR);
	phcon_remove(gtr1_WheelConstraintBL);
	phcon_remove(gtr1_WheelConstraintBR);

	phent_settype(gtr1_pCarObj, 0, 0);
	phent_settype(gtr1_pWheelFL, 0, 0);
	phent_settype(gtr1_pWheelFR, 0, 0);
	phent_settype(gtr1_pWheelBL, 0, 0);
	phent_settype(gtr1_pWheelBR, 0, 0);
	
	ent_remove(gtr1_pCarObj);
	gtr1_pCarObj = NULL;
	ent_remove(gtr1_pWheelFL);
	gtr1_pWheelFL = NULL;
	ent_remove(gtr1_pWheelFR);
	gtr1_pWheelFR = NULL;
	ent_remove(gtr1_pWheelBL);
	gtr1_pWheelBL = NULL;
	ent_remove(gtr1_pWheelBR);
	gtr1_pWheelBR = NULL;
}
*/