//*******************************************************************
// the "Shrink Me"/"Grow Me" button
//*******************************************************************


// button constants
var TR1_SHRINK_BTN_SANDBOX_X;
var TR1_SHRINK_BTN_SANDBOX_Y;
var TR1_SHRINK_BTN_SANDBOX_Z;

var TR1_GROW_BTN_SANDBOX_X;
var TR1_GROW_BTN_SANDBOX_Y;
var TR1_GROW_BTN_SANDBOX_Z;

var TR1_BTN_MIN_SCALE;
var TR1_BTN_CLICK_SCALE;

var TR1_BTN_HAPTICS_SIZE_BASE;
var TR1_BTN_HAPTICS_STIFFNESS;
var TR1_BTN_CLICK_HAPTICS_STIFFNESS;


// object local vars, stored in the entity's skills
// NOTE: skill1-skill32 are reserved for the object system
define m_scale,		skill33;		// base scale of the button



//*****************************************************
// Entity Action for the "Shrink Me"/"Grow Me" button
//*****************************************************
action tr1_aShrinkGrowBehavior
{
	var l_pos[3];
	var l_distToObj;
	var l_scale;
	
	var l_lastPressed = 0;
	
	my.shadow = off;
	my.passable = on;
	my.invisible = off;
	
	while (1)
	{
		gtr1_bButtonPressed = 0;
		gtr1_bButtonDown = 0;
		gtr1_bButtonUp = 0;

		if ((gtr1_CameraMode != TR1_CAMERA_MODE_PICKUP) && (gtr1_CameraMode != TR1_CAMERA_MODE_DRIVING))
		{	// not in a mode that displays the button
			my.invisible = on;
		}
		else
		{
			my.invisible = off;
		
			// position the button relative to the camera
			if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
			{
				vec_set(l_pos, vector(TR1_SHRINK_BTN_SANDBOX_X, TR1_SHRINK_BTN_SANDBOX_Y, TR1_SHRINK_BTN_SANDBOX_Z));
			}
			else
			{
				vec_set(l_pos, vector(TR1_GROW_BTN_SANDBOX_X, TR1_GROW_BTN_SANDBOX_Y, TR1_GROW_BTN_SANDBOX_Z));
			}
			
			vec_rotate(l_pos, camera.pan);
			vec_add(l_pos, camera.x);
			
			vec_set(my.x, l_pos);
		
			vec_set(my.pan, camera.pan);

			// check rollover collision
			
			// get the cursor's entity ptr
			tr1_Object_GetEntity(gtr1_CursorID);
			gtr1_pAltIDEntity = gtr1_pCurIDEntity;

			l_distToObj = vec_dist(gtr1_pAltIDEntity.x, my.x);
			if (l_distToObj <= my.m_objPickupDist)
			{	// hit this object
				
				// adjust scale to match how far pushed
				l_scale = l_distToObj / my.m_objPickupDist;
				if (l_scale < TR1_BTN_MIN_SCALE)
				{
					l_scale = TR1_BTN_MIN_SCALE;
				}
				
				if (l_scale <= TR1_BTN_CLICK_SCALE)
				{	// we have pressed the button down
					gtr1_bButtonPressed = 1;
					//my.m_hapticsRadius = TR1_BTN_HAPTICS_SIZE_BASE * TR1_BTN_CLICK_SCALE;
					my.m_hapticsStiffness = TR1_BTN_CLICK_HAPTICS_STIFFNESS;
				}
				else
				{
					//my.m_hapticsRadius = TR1_BTN_HAPTICS_SIZE_BASE;
					my.m_hapticsStiffness = TR1_BTN_HAPTICS_STIFFNESS;
				}

				l_scale *= my.m_scale;
			}
			else
			{
				l_scale = my.m_scale;
				//my.m_hapticsRadius = TR1_BTN_HAPTICS_SIZE_BASE;
				my.m_hapticsStiffness = TR1_BTN_HAPTICS_STIFFNESS;
			}
			
			my.scale_x = l_scale;
			my.scale_y = l_scale;
			my.scale_z = l_scale;

			// set the button flags
			if (l_lastPressed == 0)
			{
				if (gtr1_bButtonPressed != 0)
				{	// button just went down
					gtr1_bButtonDown = 1;
				}
			}
			else
			{
				if (gtr1_bButtonPressed == 0)
				{	// button was just released
					gtr1_bButtonUp = 1;
				}
			}
			
			if (gtr1_bButtonUp != 0)
			{	// just released the button
				if (gtr1_CameraMode == TR1_CAMERA_MODE_PICKUP)
				{	// go from sandbox to driving mode
					gtr1_NewCamMode = TR1_CAMERA_MODE_GOTO_DRIVING;
				}
				else
				{	// go from driving to sandbox view
					gtr1_NewCamMode = TR1_CAMERA_MODE_GOTO_SANDBOX;
				}
							
				// recreate the button as the other type
				tr1_ShrinkGrow_Btn_Create();
			}
		}

		l_lastPressed = gtr1_bButtonPressed;
			
		wait(1);
	}
}


//*******************************************************
// create the "Shrink Me"/"Grow Me" button
// NOTE: by calling this routine when the button exists, 
//			you change the object type from shrink to grow 
//			and back
//*******************************************************
function tr1_ShrinkGrow_Btn_Create ()
{
	var l_objType = TR1_OBJ_TYPE_SHRINK_BTN;
	var l_objHandle;
	var l_collideSize;
	var l_nearSize;
	
	if (gtr1_ShrinkGrowBtnID < 0)
	{	// button doesn't exist
		
		// load .ini file values
		TR1_SHRINK_BTN_SANDBOX_X = tr1_dll_getShrinkBtnSandboxX();
		TR1_SHRINK_BTN_SANDBOX_Y = tr1_dll_getShrinkBtnSandboxY();
		TR1_SHRINK_BTN_SANDBOX_Z = tr1_dll_getShrinkBtnSandboxZ();

		TR1_GROW_BTN_SANDBOX_X = tr1_dll_getGrowBtnSandboxX();
		TR1_GROW_BTN_SANDBOX_Y = tr1_dll_getGrowBtnSandboxY();
		TR1_GROW_BTN_SANDBOX_Z = tr1_dll_getGrowBtnSandboxZ();
		
		TR1_BTN_MIN_SCALE = tr1_dll_getBtnMinScale();
		TR1_BTN_CLICK_SCALE = tr1_dll_getBtnClickScale();
		
		TR1_BTN_HAPTICS_SIZE_BASE = tr1_dll_getShrinkBtnHapticsSize();
		TR1_BTN_HAPTICS_STIFFNESS = tr1_dll_getButtonStiffness();
		TR1_BTN_CLICK_HAPTICS_STIFFNESS = tr1_dll_getButtonClickStiffness();
	
		// create the button, in shrink mode
		gtr1_pCurIDEntity = 	ent_create("button_shrinkme.mdl", vector(0, 0, 1000), tr1_aShrinkGrowBehavior);
		l_objHandle = handle(gtr1_pCurIDEntity);

		l_objType = TR1_OBJ_TYPE_SHRINK_BTN;
		l_collideSize = tr1_dll_getShrinkBtnCollideSize();
		l_nearSize = tr1_dll_getShrinkBtnNearSize();

		gtr1_pCurIDEntity.m_scale = tr1_dll_getShrinkBtnScale();
	}
	else
	{	// button already exists
		l_objType = tr1_Object_GetType(gtr1_ShrinkGrowBtnID);
		l_objHandle = handle(gtr1_pCurIDEntity);

		// delete the object the button is attached to
		tr1_Object_Delete(gtr1_ShrinkGrowBtnID);
		
		// get rid of the current button model entity
		ent_remove(gtr1_pCurIDEntity);
		
		if (l_objType == TR1_OBJ_TYPE_SHRINK_BTN)
		{	// was the shrink button
			gtr1_pCurIDEntity = 	ent_create("button_growme.mdl", vector(0, 0, 1000), tr1_aShrinkGrowBehavior);
			l_objHandle = handle(gtr1_pCurIDEntity);

			l_objType = TR1_OBJ_TYPE_GROW_BTN;
			l_collideSize = tr1_dll_getGrowBtnCollideSize();
			l_nearSize = tr1_dll_getGrowBtnNearSize();

			TR1_BTN_HAPTICS_SIZE_BASE = tr1_dll_getShrinkBtnHapticsSize();

			gtr1_pCurIDEntity.m_scale = tr1_dll_getGrowBtnScale();
		}
		else
		{	// was the grow button
			gtr1_pCurIDEntity = 	ent_create("button_shrinkme.mdl", vector(0, 0, 1000), tr1_aShrinkGrowBehavior);
			l_objHandle = handle(gtr1_pCurIDEntity);

			l_objType = TR1_OBJ_TYPE_SHRINK_BTN;
			l_collideSize = tr1_dll_getShrinkBtnCollideSize();
			l_nearSize = tr1_dll_getShrinkBtnNearSize();

			TR1_BTN_HAPTICS_SIZE_BASE = tr1_dll_getGrowBtnHapticsSize();

			gtr1_pCurIDEntity.m_scale = tr1_dll_getShrinkBtnScale();
		}
		
	}

	gtr1_ShrinkGrowBtnID = tr1_Object_New(l_objType);
	if (gtr1_ShrinkGrowBtnID < 0)
	{	// error... no objects left
		return(-1);
	}
	
	// attach the button to an object
	tr1_Object_Set(gtr1_ShrinkGrowBtnID, l_objHandle, l_objType);
	tr1_Object_SetCollideInfo(gtr1_ShrinkGrowBtnID, l_collideSize, l_nearSize, 0);
	tr1_Object_SetHapticsSphere(gtr1_ShrinkGrowBtnID, TR1_BTN_HAPTICS_SIZE_BASE, TR1_BTN_HAPTICS_STIFFNESS);

	return(gtr1_ShrinkGrowBtnID);
}
